import Blob from "cross-blob";
export declare class CollisionObject {
    mask: number;
    /** -x/-z corner of 2d bounding box, expanded by 0.1 feet. Used as the origin of the heightmap grid */
    aabb_start: [number, number];
    /** +x/+z corner of 2d bounding box, expanded by 0.1 feet */
    aabb_end: [number, number];
    bounds: CollisionBoundary[];
    /** What happens when this object's floor is walked on */
    floor_type: FloorType;
    /** Where the player gets sent if drowning on this object's floor. A value of -1 is a game over. */
    drown_target: number;
    /** An index of another object where splashing effects are displayed */
    water_splash_object: number;
    floor_material: FloorMaterial;
    zone: number;
    inner_tile_size: number;
    outer_tile_size: number;
    outer_grid_width: number;
    outer_grid_height: number;
    inner_grid_size: number;
    heightmap_grid: Array<number[] | undefined>;
    get_heightmap_y(x: number, z: number, expand?: boolean): number;
    to_mesh(object_index?: number, line_mode?: boolean): {
        vertices: number[];
        indices: number[];
        types: number[];
    };
}
export interface CollisionBoundary {
    /** Location of the upper left corner of the wall/boundary */
    origin: [number, number, number];
    matrix: CollisionMatrix;
    width: number;
    height: number;
    /** Z-coordinate of the right edge of the wall/boundary minus the z-coordinate of the left edge */
    z_size: number;
    to_left: number | undefined;
    to_right: number | undefined;
}
export declare type CollisionMatrix = [number, number, number, number, number, number, number, number, number, number, number, number];
export declare enum FloorType {
    None = -1,
    Normal = 0,
    SlowWalk = 1,
    Drown = 2
}
export declare enum FloorMaterial {
    Dirt = 0,
    Grass = 1,
    Lava = 2,
    Metal = 3,
    MetalGrate = 4,
    Muck = 5,
    Stone = 6,
    Treasure = 7,
    Water = 8,
    Wood = 9,
    WoodBridge = 10,
    FastWater = 11,
    LooseRock = 12,
    Leaf = 13,
    Flower = 14,
    Pollen = 15,
    Coal = 16,
    StrawRoof = 17,
    Twigs = 18,
    Bone = 19
}
export declare class CollisionChunk {
    objects: CollisionObject[];
    id: number;
    constructor(objects: CollisionObject[], id?: number);
    static from_blob(blob: Blob): Promise<CollisionChunk>;
    to_blob(): Blob;
}
