import Blob from "cross-blob";
export declare enum TableFieldType {
    String = 0,
    Uint32 = 1,
    Int32 = 2,
    Uint16 = 3,
    Int16 = 4,
    Uint8 = 5,
    Int8 = 6,
    Float = 7
}
declare type TableField<T extends TableFieldType> = T extends TableFieldType.String ? string | null : number;
export declare type TableRowOf<T extends readonly TableFieldType[]> = {
    -readonly [P in keyof T]: T[P] extends TableFieldType ? TableField<T[P]> : never;
};
export declare class TableChunk<T extends readonly TableFieldType[] = TableFieldType[]> {
    readonly format: T;
    entries: TableRowOf<T>[];
    constructor(format: T, entries: TableRowOf<T>[]);
    static from_blob<T extends readonly TableFieldType[]>(blob: Blob, format: T): Promise<TableChunk<T>>;
    to_blob(): Blob;
    static get_format_size(format: readonly TableFieldType[]): number;
}
export declare namespace TableFormats {
    const string_table: TableFieldType[];
    /**
     * used by table 0
     */
    const item_table: readonly [TableFieldType.String, TableFieldType.Int32, TableFieldType.Int32, TableFieldType.Int32, TableFieldType.Int32, TableFieldType.Int32, TableFieldType.Int32, TableFieldType.Int16, TableFieldType.Uint8, TableFieldType.Uint8, TableFieldType.String, TableFieldType.String, TableFieldType.String, TableFieldType.String, TableFieldType.String];
    /**
     * used by tables in files of type 1000
     */
    const dialogue_table: readonly [TableFieldType.String, TableFieldType.String, TableFieldType.Uint32];
    const world_info: readonly [TableFieldType.Int32, TableFieldType.Int32, TableFieldType.Int32, TableFieldType.Int32, TableFieldType.Int32];
}
export {};
