import Blob from "cross-blob";
import { ChunkFile } from "./chunk_file.js";
import { SectorMap } from "./sector_map.js";
/**
 * A class for reading and modifying the GAMEFILE.DAT file on the game disc,
 * as well as some sub-files within GAMEFILE.DAT.
 */
export declare class Gamefile {
    #private;
    /**
     *
     * @param blob
     * @param sector_map A listing of the locations of files within this blob. If not passed in, will try to guess based on the size of the blob.
     */
    constructor(blob: Blob, sector_map?: SectorMap);
    copy(): Gamefile;
    get num_files(): number;
    get_file(index: number): Blob;
    replace_file(index: number, blob: Blob | undefined): this;
    get_chunk_file(index: number): Promise<ChunkFile>;
    replace_chunk_file(index: number, file: ChunkFile): this;
    get sector_map(): SectorMap;
    get blob(): Blob;
    static from_iso(iso_blob: Blob): Promise<Gamefile>;
    patch_iso(iso_blob: Blob): Promise<Blob>;
    [Symbol.iterator](): IterableIterator<Blob>;
    static TRANSITION_SIZE_LIMIT: number;
    static EXTERIOR_SIZE_LIMIT: number;
}
