import Blob from 'cross-blob';
;
export function guess_sector_map(file_size) {
    if (file_size == 1902385152)
        return retail_map;
    if (file_size == 1902393344)
        return oct13_map;
    if (file_size == 156485632)
        return demo_map;
    throw new Error("Could not guess sector map based on file size - please input one manually");
}
async function compare_pattern(blob, location, pattern) {
    let slice = new Uint8Array(await blob.slice(location, location + pattern.length).arrayBuffer());
    for (let i = 0; i < slice.length; i++) {
        if (pattern[i] != slice[i])
            return false;
    }
    return true;
}
async function get_sector_map_location(executable_blob) {
    if (await compare_pattern(executable_blob, 0xf4f4, [0x2d, 0x00, 0x07, 0x3c, 0x9a, 0x05, 0x08, 0x24, 0x18, 0x76, 0xe7, 0x24, 0xc8, 0xbc, 0xc6, 0x24])) {
        return {
            location: 0x1d7698,
            size: 1434
        };
    }
    throw new Error("Could not get sector map location for executable file");
}
export async function import_sector_map(executable_blob) {
    let { location, size } = await get_sector_map_location(executable_blob);
    let dv = new DataView(await executable_blob.slice(location, location + size * 8).arrayBuffer());
    let sectors = new Array(size);
    let sizes = new Array(size);
    for (let i = 0; i < size; i++) {
        sectors[i] = dv.getInt32(i * 8, true);
        sizes[i] = dv.getInt32(i * 8 + 4, true);
    }
    return { sectors, sizes };
}
export async function export_sector_map(executable_blob, sector_map) {
    let { location, size } = await get_sector_map_location(executable_blob);
    if (size != sector_map.sectors.length) {
        throw new Error("Attempted to export sector map to executable with non-matching length (sector map has " + sector_map.sectors.length + " files, executable expects " + size + " files)");
    }
    let dv = new DataView(new ArrayBuffer(size * 8));
    for (let i = 0; i < size; i++) {
        dv.setInt32(i * 8, sector_map.sectors[i], true);
        dv.setInt32(i * 8 + 4, sector_map.sizes[i], true);
    }
    return new Blob([
        executable_blob.slice(0, location),
        dv.buffer,
        executable_blob.slice(location + size * 8, executable_blob.size)
    ]);
}
const retail_map = {
    sectors: [0x0, 0x3A, 0x74, 0xAE, 0xF8, 0xF8, 0xF8, 0xF8, 0x260, 0x260, 0x545, 0x6F8, 0x8B0, 0xA5C, 0xC17, 0xDD8, 0xF94, 0x1147, 0x1300, 0x1493, 0x1649, 0x181F, 0x19AD, 0x1B6D, 0x1D03, 0x1EA6, 0x2049, 0x21FE, 0x23A0, 0x254A, 0x26EA, 0x2876, 0x2A01, 0x2B64, 0x2D12, 0x2E8C, 0x3019, 0x31D2, 0x3375, 0x3518, 0x36AB, 0x384F, 0x3A0F, 0x3BBF, 0x3D5F, 0x3F16, 0x40CD, 0x427F, 0x4444, 0x45ED, 0x478A, 0x492A, 0x4AB6, 0x4C5D, 0x4DFE, 0x4F8D, 0x5122, 0x52F0, 0x54C6, 0x567B, 0x57F3, 0x59A4, 0x5B64, 0x5D08, 0x5EC8, 0x6088, 0x6248, 0x63F4, 0x65C7, 0x676A, 0x6908, 0x6AA8, 0x6C7B, 0x6E4E, 0x7019, 0x719D, 0x735E, 0x7511, 0x76C4, 0x786D, 0x7A20, 0x7BD4, 0x7D8B, 0x7F3A, 0x80ED, 0x82AB, 0x843E, 0x85F4, 0x87BA, 0x8948, 0x8B03, 0x8C99, 0x8E3C, 0x8FE0, 0x9195, 0x9336, 0x94DE, 0x967E, 0x980A, 0x9995, 0x9AF8, 0x9CAE, 0x9E28, 0x9FB5, 0xA16E, 0xA30B, 0xA4A8, 0xA63B, 0xA7DF, 0xA99F, 0xAB4F, 0xACEF, 0xAEA4, 0xB059, 0xB20A, 0xB3C0, 0xB569, 0xB706, 0xB8A6, 0xBA32, 0xBBD9, 0xBD79, 0xBF08, 0xC09D, 0xC26B, 0xC42E, 0xC5E3, 0xC75B, 0xC909, 0xCAC6, 0xCC6A, 0xCE2A, 0xCFEA, 0xD1AA, 0xD349, 0xD51C, 0xD6BF, 0xD85D, 0xD9FD, 0xDBD0, 0xDDA3, 0xDF6E, 0xE0F2, 0xE2B3, 0xE466, 0x10BAE, 0x118AD, 0x11CFA, 0x1211F, 0x12544, 0x12905, 0x12E7E, 0x1323F, 0x1368C, 0x13A25, 0x13E36, 0x141E3, 0x14590, 0x14A55, 0x14E3E, 0x1515F, 0x15520, 0x15981, 0x15D92, 0x1627F, 0x1676C, 0x16BCD, 0x17092, 0x17543, 0x17990, 0x17DDD, 0x18216, 0x18663, 0x18A24, 0x18F9D, 0x1935E, 0x1971F, 0x19AE0, 0x19E79, 0x1A23A, 0x1A64B, 0x1ADB8, 0x1B525, 0x1BC92, 0x1BF27, 0x1C1BC, 0x1C929, 0x1D096, 0x1D42F, 0x1D7C8, 0x1DF35, 0x1E256, 0x1E437, 0x1E974, 0x1ED85, 0x1F196, 0x1F43F, 0x1F850, 0x1FD8D, 0x203A6, 0x209BF, 0x20D80, 0x211CD, 0x215F2, 0x21A17, 0x21DD8, 0x22351, 0x22712, 0x22B5F, 0x22FE8, 0x233F9, 0x23882, 0x23D33, 0x241F8, 0x24645, 0x24966, 0x24E17, 0x25278, 0x25689, 0x25AD6, 0x25FC3, 0x26424, 0x268E9, 0x26D9A, 0x271E7, 0x27634, 0x27A6D, 0x27EBA, 0x28343, 0x28754, 0x28BA1, 0x28FEE, 0x2949F, 0x29838, 0x29BF9, 0x2A00A, 0x2A457, 0x2A8A4, 0x2ACF1, 0x2AF86, 0x2B21B, 0x2B668, 0x2BAB5, 0x2BE4E, 0x2C1E7, 0x2C954, 0x2CC75, 0x2D126, 0x2D663, 0x2DA74, 0x2DE85, 0x2E12E, 0x2E53F, 0x2EA7C, 0x2F095, 0x2F6AE, 0x2FB37, 0x2FB5E, 0x2FB86, 0x2FB9C, 0x2FBB6, 0x2FBDD, 0x2FC12, 0x2FC31, 0x2FC59, 0x2FC79, 0x2FC8E, 0x2FCAE, 0x2FCD6, 0x2FCFC, 0x2FD1B, 0x2FD37, 0x2FD62, 0x2FD79, 0x2FD91, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FE13, 0x2FE27, 0x2FE3E, 0x2FE73, 0x2FEA2, 0x2FEC5, 0x2FF02, 0x2FF23, 0x2FF58, 0x2FF5A, 0x2FF5C, 0x2FF5E, 0x2FF60, 0x2FF96, 0x2FFC8, 0x2FFF9, 0x30024, 0x3005D, 0x300BB, 0x3010F, 0x30173, 0x301AA, 0x301E0, 0x30217, 0x3024F, 0x30286, 0x302B6, 0x302E6, 0x3031C, 0x30350, 0x3037C, 0x303B5, 0x303E4, 0x30417, 0x30419, 0x3044C, 0x3047E, 0x304E3, 0x304E5, 0x30519, 0x30587, 0x305DC, 0x30627, 0x30629, 0x306EB, 0x3071E, 0x30751, 0x307A6, 0x307B5, 0x307D7, 0x307DE, 0x307F4, 0x30830, 0x3086C, 0x308A8, 0x308B7, 0x308D7, 0x30905, 0x3093D, 0x30954, 0x30976, 0x30982, 0x3098F, 0x3099B, 0x309C2, 0x309F2, 0x30A00, 0x30A2C, 0x30A4E, 0x30A7F, 0x30A84, 0x30A99, 0x30AB1, 0x30AB6, 0x30ACB, 0x30AE3, 0x30AF6, 0x30B29, 0x30B4C, 0x30B63, 0x30B76, 0x30BA7, 0x30BB6, 0x30BBD, 0x30BEE, 0x30C21, 0x30C4E, 0x30C80, 0x30C9F, 0x30CAF, 0x30CB5, 0x30CE5, 0x30D17, 0x30D48, 0x30D60, 0x30D91, 0x30DBB, 0x30DE9, 0x30DF6, 0x30E0E, 0x30E2C, 0x30E4A, 0x30E78, 0x30E7D, 0x30E99, 0x30EAE, 0x30ECA, 0x30EE6, 0x30EEB, 0x30EEF, 0x30EF3, 0x30EF8, 0x30EFE, 0x30F02, 0x30F18, 0x30F2B, 0x30F47, 0x30F67, 0x30F8A, 0x30FAF, 0x30FCE, 0x30FF2, 0x31010, 0x3102D, 0x3104B, 0x31069, 0x31087, 0x310A5, 0x310C3, 0x310E6, 0x31109, 0x3112C, 0x3114D, 0x31173, 0x31199, 0x311BC, 0x311E0, 0x31204, 0x31230, 0x3124C, 0x31264, 0x3127C, 0x3129D, 0x312B8, 0x312D1, 0x312D3, 0x314C2, 0x314C3, 0x3151F, 0x31520, 0x3157C, 0x3157D, 0x31740, 0x31741, 0x317D3, 0x317D4, 0x3184F, 0x318F0, 0x3199C, 0x31A44, 0x31AE8, 0x31B9A, 0x31C49, 0x31D09, 0x31DAD, 0x31E7D, 0x31F56, 0x3202B, 0x32103, 0x321E6, 0x322BE, 0x3239D, 0x3243D, 0x3243E, 0x3243F, 0x32440, 0x32441, 0x32442, 0x32443, 0x3259E, 0x3259F, 0x325E9, 0x325EF, 0x32692, 0x3269F, 0x327F3, 0x3282A, 0x328DB, 0x329C6, 0x32A82, 0x32B12, 0x32B6D, 0x32BD3, 0x32C61, 0x32D3F, 0x32E1D, 0x32E7D, 0x32E83, 0x32E8A, 0x32E92, 0x32EA4, 0x32EB2, 0x32EBE, 0x32ECA, 0x32ED0, 0x32ED7, 0x32EE8, 0x32EFE, 0x32F07, 0x32F16, 0x32F25, 0x32F3D, 0x32F48, 0x32F53, 0x32F62, 0x32F78, 0x32F87, 0x32F96, 0x32FA1, 0x32FB3, 0x32FC5, 0x32FCC, 0x32FDA, 0x32FE9, 0x32FF6, 0x32FFE, 0x3300E, 0x3301E, 0x33028, 0x34A48, 0x34AC9, 0x34B4A, 0x34BCB, 0x34C4C, 0x34CCD, 0x34D4E, 0x34DCF, 0x34E50, 0x34ED1, 0x34F52, 0x34FD3, 0x35054, 0x350D5, 0x35156, 0x351D7, 0x35258, 0x352D9, 0x3531A, 0x3539B, 0x3541C, 0x3549D, 0x3551E, 0x3559F, 0x35620, 0x356A1, 0x35722, 0x357A3, 0x35824, 0x358A5, 0x35926, 0x359A7, 0x35A28, 0x35AA9, 0x35B2A, 0x35BAB, 0x35C2C, 0x35CAD, 0x35D2E, 0x35DAF, 0x35E30, 0x35EB1, 0x35F32, 0x35FB3, 0x36034, 0x360B5, 0x36136, 0x361B7, 0x36238, 0x362B9, 0x3633A, 0x363BB, 0x3643C, 0x364BD, 0x3653E, 0x365BF, 0x36640, 0x366C1, 0x36742, 0x367C3, 0x36844, 0x368C5, 0x36946, 0x369C7, 0x36A48, 0x36AC9, 0x36B4A, 0x36BCB, 0x36C4C, 0x36CCD, 0x36D4E, 0x36DCF, 0x36E50, 0x36ED1, 0x36F52, 0x36FD3, 0x37054, 0x370D5, 0x37156, 0x371D7, 0x37258, 0x372D9, 0x3735A, 0x373DB, 0x3745C, 0x374DD, 0x3755E, 0x375DF, 0x37660, 0x376E1, 0x37762, 0x377E3, 0x37864, 0x378E5, 0x37966, 0x379E7, 0x37A68, 0x37AE9, 0x37B6A, 0x37BEB, 0x37C6C, 0x37CED, 0x38115, 0x3859D, 0x38905, 0x38BC9, 0x38E34, 0x38F7D, 0x3975A, 0x39F9B, 0x3A3C3, 0x3A84B, 0x3ABB3, 0x3AE77, 0x3B0E2, 0x3B50A, 0x3B992, 0x3BCFA, 0x3BFBE, 0x3C229, 0x3CA6C, 0x3CAEA, 0x3CB68, 0x3D365, 0x3D523, 0x3D678, 0x3DC65, 0x3E191, 0x3E7F6, 0x3ECB4, 0x3F457, 0x3F55C, 0x3F5E4, 0x3F66C, 0x3F787, 0x3F8A2, 0x3F92A, 0x3FDFD, 0x404F1, 0x4069C, 0x40842, 0x40AA8, 0x40DE9, 0x40ECD, 0x40F88, 0x41402, 0x41C88, 0x41CF1, 0x420B7, 0x4247D, 0x429EB, 0x42AC5, 0x42BD1, 0x42CF6, 0x42D9B, 0x43406, 0x43B3A, 0x43CC6, 0x43DEA, 0x44F20, 0x4516F, 0x453C1, 0x456AF, 0x4599D, 0x45AD7, 0x45C13, 0x4609E, 0x462E3, 0x4646F, 0x46914, 0x46B1F, 0x46C2B, 0x47082, 0x4767C, 0x47BDF, 0x47CA1, 0x4815A, 0x48C62, 0x4901B, 0x49258, 0x49449, 0x4A4AB, 0x4B20E, 0x4B271, 0x4B46F, 0x4B66D, 0x4B878, 0x4BACA, 0x4BCDD, 0x4BE84, 0x4C091, 0x4C29E, 0x4C434, 0x4C64B, 0x4C758, 0x4C891, 0x4DB83, 0x4DCBF, 0x4DF96, 0x4E07F, 0x4E5A8, 0x4F19E, 0x4F383, 0x4F568, 0x4F833, 0x4F91C, 0x4FE31, 0x502C4, 0x5051A, 0x5065D, 0x50A14, 0x50D9F, 0x51236, 0x517A0, 0x52683, 0x52A28, 0x52F8F, 0x53617, 0x5389F, 0x539EB, 0x5452B, 0x545B0, 0x54677, 0x54884, 0x54FAE, 0x55327, 0x553A8, 0x55430, 0x55799, 0x55989, 0x55B7F, 0x55D71, 0x55F04, 0x5612D, 0x56245, 0x5635D, 0x56475, 0x5658D, 0x566A5, 0x566A6, 0x56B39, 0x56C51, 0x56D69, 0x56E81, 0x56F99, 0x570B1, 0x583AA, 0x58839, 0x58944, 0x58B66, 0x58FF1, 0x5936A, 0x59483, 0x5959C, 0x59791, 0x59BA7, 0x59C30, 0x59CB9, 0x5A035, 0x5A1D7, 0x5A379, 0x5A78E, 0x5A97E, 0x5AB6E, 0x5AD5E, 0x5AF4E, 0x5BA3F, 0x5C25F, 0x5C8BB, 0x5CE58, 0x5D3DD, 0x5D4C4, 0x5DAD0, 0x5E12C, 0x5E1E2, 0x5E8B1, 0x5E96B, 0x5EA25, 0x5EADF, 0x5EF11, 0x5F3A5, 0x5F822, 0x5FCCD, 0x5FFF8, 0x5FFF9, 0x60462, 0x60608, 0x60916, 0x61B8C, 0x61DF8, 0x6213C, 0x6224E, 0x62352, 0x62548, 0x6265A, 0x6278C, 0x62880, 0x6298C, 0x62D33, 0x62EB0, 0x6303D, 0x6314F, 0x63281, 0x63A40, 0x64657, 0x64CEC, 0x64F42, 0x6503F, 0x6513C, 0x6528F, 0x65CA5, 0x6620C, 0x6653A, 0x66803, 0x6688B, 0x669E0, 0x66C80, 0x67D5D, 0x67DEB, 0x67FA7, 0x68154, 0x6826F, 0x6853F, 0x68626, 0x6870D, 0x689CD, 0x68AD2, 0x68B4F, 0x68BCC, 0x68BCD, 0x69615, 0x69616, 0x69694, 0x6A32D, 0x6A3EE, 0x6A4AD, 0x6A510, 0x6A9C6, 0x6AAE1, 0x6ABFC, 0x6AD17, 0x6AE32, 0x6AF4D, 0x6B068, 0x6B162, 0x6B25C, 0x6B356, 0x6B450, 0x6B54A, 0x6B644, 0x6B72B, 0x6B812, 0x6B8F9, 0x6B9E0, 0x6BAC7, 0x6BBAE, 0x6BC95, 0x6BD7C, 0x6CCB6, 0x6CEAA, 0x6D09A, 0x6D177, 0x6D254, 0x6D2DF, 0x6D3A5, 0x6D54E, 0x6D959, 0x6DC37, 0x6DD4D, 0x6DE5F, 0x6E0E1, 0x6E7EC, 0x6E8DF, 0x6EB9F, 0x6EC95, 0x6F0B9, 0x6F3B4, 0x6F6AF, 0x6F9AA, 0x6FB31, 0x70ED3, 0x70F7B, 0x710A2, 0x711C8, 0x712AE, 0x713CB, 0x714CF, 0x715D3, 0x71853, 0x71971, 0x71C88, 0x71EA2, 0x72084, 0x72361, 0x73703, 0x737AB, 0x738D2, 0x739F8, 0x73ADE, 0x73BFB, 0x73CFF, 0x73E03, 0x74083, 0x741A1, 0x744B8, 0x746D2, 0x748B4, 0x74B91, 0x74FB0, 0x76139, 0x7720C, 0x77641, 0x77763, 0x77D2C, 0x78C66, 0x79E74, 0x79F69, 0x7A0A6, 0x7A161, 0x7A29A, 0x7A59E, 0x7A8CE, 0x7AADB, 0x7ADD8, 0x7B0A9, 0x7B188, 0x7B386, 0x7BB8B, 0x7BBBC, 0x7BFCA, 0x7C103, 0x7C23C, 0x7C5F9, 0x7C709, 0x7C819, 0x7CDAD, 0x7CE73, 0x7CF39, 0x7D2E6, 0x7D4A7, 0x7D668, 0x7DCAC, 0x7E04F, 0x7E3F2, 0x7E8D8, 0x7E9DC, 0x7EAE0, 0x7FD00, 0x7FDDD, 0x7FECA, 0x80024, 0x801DB, 0x8037A, 0x804C9, 0x80767, 0x80949, 0x80B5D, 0x80D73, 0x80F6A, 0x8118F, 0x813A7, 0x8154E, 0x81604, 0x81805, 0x81A8E, 0x81C70, 0x81E49, 0x81FEA, 0x8222C, 0x8246A, 0x8266C, 0x828AB, 0x829BF, 0x82AFC, 0x82BFD, 0x82CFF, 0x82DF5, 0x82F0B, 0x8301F, 0x83077, 0x8405C, 0x842D5, 0x8454E, 0x84853, 0x84ACC, 0x84BD0, 0x84C5D, 0x84D8A, 0x8507D, 0x85370, 0x85BF6, 0x85C80, 0x85EE0, 0x8641D, 0x86522, 0x8665E, 0x8679A, 0x8692E, 0x87852, 0x879BA, 0x87B22, 0x8876A, 0x89D0A, 0x8A09F, 0x8B7C3, 0x8C4BF, 0x8C4ED, 0x8D058, 0x8D32B, 0x8D725, 0x8DB21, 0x8E066, 0x8E5AB, 0x8EC2F, 0x8FF4A, 0x9122D, 0x91352, 0x9147F, 0x915AA, 0x916E0, 0x927E0, 0x927E1, 0x936F2, 0x9384A, 0x939A2, 0x93CF0, 0x93E02, 0x93F1F, 0x9403C, 0x94145, 0x94350, 0x946FC, 0x94AE4, 0x94D6D, 0x94E71, 0x94F75, 0x9506B, 0x95170, 0x95304, 0x95424, 0x95533, 0x95653, 0x959CA, 0x95AC0, 0x95AC1, 0x95C36, 0x9632B, 0x96745, 0x969EB, 0x96E23, 0x96F7C, 0x97107, 0x97247, 0x97E8F, 0x981A3, 0x986BD, 0x986BE, 0x98927, 0x98A3D, 0x98B53, 0x990A4, 0x991A9, 0x992AE, 0x99992, 0x99DC6, 0x99ED0, 0x99FDA, 0x9A389, 0x9A464, 0x9A53F, 0x9A61A, 0x9A6F5, 0x9AE2D, 0x9AFAF, 0x9B722, 0x9B826, 0x9BAD4, 0x9BBD9, 0x9BCF4, 0x9BE12, 0x9BFE4, 0x9C0FD, 0x9C21C, 0x9C3BF, 0x9C49E, 0x9C5B7, 0x9C6D3, 0x9CCB2, 0x9D22F, 0x9D363, 0x9D4BA, 0x9D5BF, 0x9D6DF, 0x9D7C6, 0x9DC4D, 0x9EC05, 0x9F13B, 0x9F4CA, 0xA0431, 0xA08C6, 0xA0DDB, 0xA1115, 0xA156F, 0xA16B2, 0xA18A2, 0xA19BB, 0xA1B6E, 0xA1D2F, 0xA2EA8, 0xA3049, 0xA32FD, 0xA35B1, 0xA38F7, 0xA3A68, 0xA3D15, 0xA3FC2, 0xA4165, 0xA4308, 0xA4364, 0xA43C0, 0xA4529, 0xA4699, 0xA4946, 0xA4BF3, 0xA4D99, 0xA4F3F, 0xA5156, 0xA51CD, 0xA549E, 0xA57A8, 0xA5A55, 0xA5D02, 0xA5D5E, 0xA5F08, 0xA60B2, 0xA6117, 0xA6177, 0xA631A, 0xA64BD, 0xA6565, 0xA65E0, 0xA7467, 0xA7592, 0xA76CA, 0xA7A56, 0xA7B60, 0xA7C6A, 0xA7D96, 0xA7EC2, 0xA8411, 0xA851E, 0xA862F, 0xA873C, 0xA8849, 0xA96C4, 0xA9BC0, 0xAA0BE, 0xAA395, 0xAAA24, 0xAABAF, 0xAAD3A, 0xAAD3B, 0xAB75D, 0xABB1C, 0xABB7B, 0xABBD9, 0xABDDF, 0xABDE0, 0xACDBF, 0xADBC8, 0xAEF30, 0xAFF70, 0xB10F6, 0xB20B0, 0xB3605, 0xB388F, 0xB3890, 0xB455A, 0xB4668, 0xB4B10, 0xB4EAA, 0xB5AC4, 0xB5C00, 0xB5F1C, 0xB627B, 0xB6707, 0xB68FB, 0xB6EF6, 0xB7542, 0xB7637, 0xB7BA5, 0xB7D4F, 0xB7EEB, 0xB8715, 0xB8C69, 0xB8EAA, 0xB90D8, 0xB9142, 0xB9285, 0xB9705, 0xB9B31, 0xB9D94, 0xBA08F, 0xBA38A, 0xBA712, 0xBA9B6, 0xBAEA0, 0xBB365, 0xBB7DE, 0xBBCF3, 0xBC124, 0xBC56F, 0xBC9DE, 0xBCDAA, 0xBD142, 0xBD6AD, 0xBDC75, 0xBDF2B, 0xBE304, 0xBE7A1, 0xBEB23, 0xBEE22, 0xBF017, 0xBF53C, 0xC043B, 0xC04F9, 0xC05BD, 0xC064D, 0xC075C, 0xC086B, 0xC097A, 0xC0A89, 0xC0B8D, 0xC0C91, 0xC0D95, 0xC0EAA, 0xC0FBF, 0xC10D4, 0xC11E9, 0xC15E5, 0xC1721, 0xC1B69, 0xC229C, 0xC2524, 0xC27A7, 0xC28CE, 0xC312B, 0xC42F7, 0xC49A1, 0xC4DCA, 0xC4EEF, 0xC5936, 0xC59AA, 0xC5EAD, 0xC5F0F, 0xC5F78, 0xC64E0, 0xC65EF, 0xC66F3, 0xC794A, 0xC7CE7, 0xC7F74, 0xC831E, 0xC841F, 0xC8513, 0xC85F2, 0xC8801, 0xC8905, 0xC8E13, 0xC9093, 0xC9313, 0xC9593, 0xC96A8, 0xC9988, 0xCA880, 0xCAF7D, 0xCB751, 0xCC7F2, 0xCCC28, 0xCCE20, 0xCD33B, 0xCD55E, 0xCE55B, 0xCEB0D, 0xCEE15, 0xCF01F, 0xCF1EB, 0xD0413, 0xD089A, 0xD099B, 0xD0B87, 0xD0DF9, 0xD0E79, 0xD1779, 0xD1CF6, 0xD2943, 0xD2D64, 0xD30D5, 0xD327C, 0xD35D4, 0xD38F9, 0xD3A0B, 0xD3E06, 0xD43DC, 0xD4830, 0xD50D3, 0xD5F2C, 0xD60EA, 0xD64FA, 0xD68FD, 0xD6CF7, 0xD7107, 0xD74B4, 0xD78B4, 0xD7B9D, 0xD7D81, 0xD8D3D, 0xD99C1, 0xDA016, 0xDA230, 0xDA3B0, 0xDB6A6, 0xDC3B4, 0xDC55A, 0xDC5B6, 0xDC61A, 0xDC773, 0xDC893, 0xDC9F9, 0xDCB93, 0xDCC94, 0xDCEA7, 0xDCF7E, 0xDD247, 0xDD46C, 0xDDBB1, 0xDE2B6, 0xDE88A, 0xDF247, 0xDF5F9, 0xDFAB0, 0xDFDA5, 0xDFEBD, 0xE0246, 0xE0360, 0xE0534, 0xE062E, 0xE077E, 0xE08A4, 0xE0C8D, 0xE0D92, 0xE102D, 0xE16A7, 0xE1A68, 0xE1E29, 0xE21EA, 0xE28C2],
    sizes: [0x3A, 0x3A, 0x3A, 0x4A, 0x0, 0x0, 0x0, 0x168, 0x0, 0x2E5, 0x1B3, 0x1B8, 0x1AC, 0x1BB, 0x1C1, 0x1BC, 0x1B3, 0x1B9, 0x193, 0x1B6, 0x1D6, 0x18E, 0x1C0, 0x196, 0x1A3, 0x1A3, 0x1B5, 0x1A2, 0x1AA, 0x1A0, 0x18C, 0x18B, 0x163, 0x1AE, 0x17A, 0x18D, 0x1B9, 0x1A3, 0x1A3, 0x193, 0x1A4, 0x1C0, 0x1B0, 0x1A0, 0x1B7, 0x1B7, 0x1B2, 0x1C5, 0x1A9, 0x19D, 0x1A0, 0x18C, 0x1A7, 0x1A1, 0x18F, 0x195, 0x1CE, 0x1D6, 0x1B5, 0x178, 0x1B1, 0x1C0, 0x1A4, 0x1C0, 0x1C0, 0x1C0, 0x1AC, 0x1D3, 0x1A3, 0x19E, 0x1A0, 0x1D3, 0x1D3, 0x1CB, 0x184, 0x1C1, 0x1B3, 0x1B3, 0x1A9, 0x1B3, 0x1B4, 0x1B7, 0x1AF, 0x1B3, 0x1BE, 0x193, 0x1B6, 0x1C6, 0x18E, 0x1BB, 0x196, 0x1A3, 0x1A4, 0x1B5, 0x1A1, 0x1A8, 0x1A0, 0x18C, 0x18B, 0x163, 0x1B6, 0x17A, 0x18D, 0x1B9, 0x19D, 0x19D, 0x193, 0x1A4, 0x1C0, 0x1B0, 0x1A0, 0x1B5, 0x1B5, 0x1B1, 0x1B6, 0x1A9, 0x19D, 0x1A0, 0x18C, 0x1A7, 0x1A0, 0x18F, 0x195, 0x1CE, 0x1C3, 0x1B5, 0x178, 0x1AE, 0x1BD, 0x1A4, 0x1C0, 0x1C0, 0x1C0, 0x19F, 0x1D3, 0x1A3, 0x19E, 0x1A0, 0x1D3, 0x1D3, 0x1CB, 0x184, 0x1C1, 0x1B3, 0x2748, 0xCFF, 0x44D, 0x425, 0x425, 0x3C1, 0x579, 0x3C1, 0x44D, 0x399, 0x411, 0x3AD, 0x3AD, 0x4C5, 0x3E9, 0x321, 0x3C1, 0x461, 0x411, 0x4ED, 0x4ED, 0x461, 0x4C5, 0x4B1, 0x44D, 0x44D, 0x439, 0x44D, 0x3C1, 0x579, 0x3C1, 0x3C1, 0x3C1, 0x399, 0x3C1, 0x411, 0x76D, 0x76D, 0x76D, 0x295, 0x295, 0x76D, 0x76D, 0x399, 0x399, 0x76D, 0x321, 0x1E1, 0x53D, 0x411, 0x411, 0x2A9, 0x411, 0x53D, 0x619, 0x619, 0x3C1, 0x44D, 0x425, 0x425, 0x3C1, 0x579, 0x3C1, 0x44D, 0x489, 0x411, 0x489, 0x4B1, 0x4C5, 0x44D, 0x321, 0x4B1, 0x461, 0x411, 0x44D, 0x4ED, 0x461, 0x4C5, 0x4B1, 0x44D, 0x44D, 0x439, 0x44D, 0x489, 0x411, 0x44D, 0x44D, 0x4B1, 0x399, 0x3C1, 0x411, 0x44D, 0x44D, 0x44D, 0x295, 0x295, 0x44D, 0x44D, 0x399, 0x399, 0x76D, 0x321, 0x4B1, 0x53D, 0x411, 0x411, 0x2A9, 0x411, 0x53D, 0x619, 0x619, 0x489, 0x27, 0x28, 0x16, 0x1A, 0x27, 0x35, 0x1F, 0x28, 0x20, 0x15, 0x20, 0x28, 0x26, 0x1F, 0x1C, 0x2B, 0x17, 0x18, 0x11, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x71, 0x14, 0x17, 0x35, 0x2F, 0x23, 0x3D, 0x21, 0x35, 0x2, 0x2, 0x2, 0x2, 0x36, 0x32, 0x31, 0x2B, 0x39, 0x5E, 0x54, 0x64, 0x37, 0x36, 0x37, 0x38, 0x37, 0x30, 0x30, 0x36, 0x34, 0x2C, 0x39, 0x2F, 0x33, 0x2, 0x33, 0x32, 0x65, 0x2, 0x34, 0x6E, 0x55, 0x4B, 0x2, 0xC2, 0x33, 0x33, 0x55, 0xF, 0x22, 0x7, 0x16, 0x3C, 0x3C, 0x3C, 0xF, 0x20, 0x2E, 0x38, 0x17, 0x22, 0xC, 0xD, 0xC, 0x27, 0x30, 0xE, 0x2C, 0x22, 0x31, 0x5, 0x15, 0x18, 0x5, 0x15, 0x18, 0x13, 0x33, 0x23, 0x17, 0x13, 0x31, 0xF, 0x7, 0x31, 0x33, 0x2D, 0x32, 0x1F, 0x10, 0x6, 0x30, 0x32, 0x31, 0x18, 0x31, 0x2A, 0x2E, 0xD, 0x18, 0x1E, 0x1E, 0x2E, 0x5, 0x1C, 0x15, 0x1C, 0x1C, 0x5, 0x4, 0x4, 0x5, 0x6, 0x4, 0x16, 0x13, 0x1C, 0x20, 0x23, 0x25, 0x1F, 0x24, 0x1E, 0x1D, 0x1E, 0x1E, 0x1E, 0x1E, 0x1E, 0x23, 0x23, 0x23, 0x21, 0x26, 0x26, 0x23, 0x24, 0x24, 0x2C, 0x1C, 0x18, 0x18, 0x21, 0x1B, 0x19, 0x2, 0x1EF, 0x1, 0x5C, 0x1, 0x5C, 0x1, 0x1C3, 0x1, 0x92, 0x1, 0x7B, 0xA1, 0xAC, 0xA8, 0xA4, 0xB2, 0xAF, 0xC0, 0xA4, 0xD0, 0xD9, 0xD5, 0xD8, 0xE3, 0xD8, 0xDF, 0xA0, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x15B, 0x1, 0x4A, 0x6, 0xA3, 0xD, 0x154, 0x37, 0xB1, 0xEB, 0xBC, 0x90, 0x5B, 0x66, 0x8E, 0xDE, 0xDE, 0x60, 0x6, 0x7, 0x8, 0x12, 0xE, 0xC, 0xC, 0x6, 0x7, 0x11, 0x16, 0x9, 0xF, 0xF, 0x18, 0xB, 0xB, 0xF, 0x16, 0xF, 0xF, 0xB, 0x12, 0x12, 0x7, 0xE, 0xF, 0xD, 0x8, 0x10, 0x10, 0xA, 0x1A20, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x41, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x428, 0x488, 0x368, 0x2C4, 0x26B, 0x149, 0x7DD, 0x841, 0x428, 0x488, 0x368, 0x2C4, 0x26B, 0x428, 0x488, 0x368, 0x2C4, 0x26B, 0x843, 0x7E, 0x7E, 0x7FD, 0x1BE, 0x155, 0x5ED, 0x52C, 0x665, 0x4BE, 0x7A3, 0x105, 0x88, 0x88, 0x11B, 0x11B, 0x88, 0x4D3, 0x6F4, 0x1AB, 0x1A6, 0x266, 0x341, 0xE4, 0xBB, 0x47A, 0x886, 0x69, 0x3C6, 0x3C6, 0x56E, 0xDA, 0x10C, 0x125, 0xA5, 0x66B, 0x734, 0x18C, 0x124, 0x1136, 0x24F, 0x252, 0x2EE, 0x2EE, 0x13A, 0x13C, 0x48B, 0x245, 0x18C, 0x4A5, 0x20B, 0x10C, 0x457, 0x5FA, 0x563, 0xC2, 0x4B9, 0xB08, 0x3B9, 0x23D, 0x1F1, 0x1062, 0xD63, 0x63, 0x1FE, 0x1FE, 0x20B, 0x252, 0x213, 0x1A7, 0x20D, 0x20D, 0x196, 0x217, 0x10D, 0x139, 0x12F2, 0x13C, 0x2D7, 0xE9, 0x529, 0xBF6, 0x1E5, 0x1E5, 0x2CB, 0xE9, 0x515, 0x493, 0x256, 0x143, 0x3B7, 0x38B, 0x497, 0x56A, 0xEE3, 0x3A5, 0x567, 0x688, 0x288, 0x14C, 0xB40, 0x85, 0xC7, 0x20D, 0x72A, 0x379, 0x81, 0x88, 0x369, 0x1F0, 0x1F6, 0x1F2, 0x193, 0x229, 0x118, 0x118, 0x118, 0x118, 0x118, 0x1, 0x493, 0x118, 0x118, 0x118, 0x118, 0x118, 0x12F9, 0x48F, 0x10B, 0x222, 0x48B, 0x379, 0x119, 0x119, 0x1F5, 0x416, 0x89, 0x89, 0x37C, 0x1A2, 0x1A2, 0x415, 0x1F0, 0x1F0, 0x1F0, 0x1F0, 0xAF1, 0x820, 0x65C, 0x59D, 0x585, 0xE7, 0x60C, 0x65C, 0xB6, 0x6CF, 0xBA, 0xBA, 0xBA, 0x432, 0x494, 0x47D, 0x4AB, 0x32B, 0x1, 0x469, 0x1A6, 0x30E, 0x1276, 0x26C, 0x344, 0x112, 0x104, 0x1F6, 0x112, 0x132, 0xF4, 0x10C, 0x3A7, 0x17D, 0x18D, 0x112, 0x132, 0x7BF, 0xC17, 0x695, 0x256, 0xFD, 0xFD, 0x153, 0xA16, 0x567, 0x32E, 0x2C9, 0x88, 0x155, 0x2A0, 0x10DD, 0x8E, 0x1BC, 0x1AD, 0x11B, 0x2D0, 0xE7, 0xE7, 0x2C0, 0x105, 0x7D, 0x7D, 0x1, 0xA48, 0x1, 0x7E, 0xC99, 0xC1, 0xBF, 0x63, 0x4B6, 0x11B, 0x11B, 0x11B, 0x11B, 0x11B, 0x11B, 0xFA, 0xFA, 0xFA, 0xFA, 0xFA, 0xFA, 0xE7, 0xE7, 0xE7, 0xE7, 0xE7, 0xE7, 0xE7, 0xE7, 0xF3A, 0x1F4, 0x1F0, 0xDD, 0xDD, 0x8B, 0xC6, 0x1A9, 0x40B, 0x2DE, 0x116, 0x112, 0x282, 0x70B, 0xF3, 0x2C0, 0xF6, 0x424, 0x2FB, 0x2FB, 0x2FB, 0x187, 0x13A2, 0xA8, 0x127, 0x126, 0xE6, 0x11D, 0x104, 0x104, 0x280, 0x11E, 0x317, 0x21A, 0x1E2, 0x2DD, 0x13A2, 0xA8, 0x127, 0x126, 0xE6, 0x11D, 0x104, 0x104, 0x280, 0x11E, 0x317, 0x21A, 0x1E2, 0x2DD, 0x41F, 0x1189, 0x10D3, 0x435, 0x122, 0x5C9, 0xF3A, 0x120E, 0xF5, 0x13D, 0xBB, 0x139, 0x304, 0x330, 0x20D, 0x2FD, 0x2D1, 0xDF, 0x1FE, 0x805, 0x31, 0x40E, 0x139, 0x139, 0x3BD, 0x110, 0x110, 0x594, 0xC6, 0xC6, 0x3AD, 0x1C1, 0x1C1, 0x644, 0x3A3, 0x3A3, 0x4E6, 0x104, 0x104, 0x1220, 0xDD, 0xED, 0x15A, 0x1B7, 0x19F, 0x14F, 0x29E, 0x1E2, 0x214, 0x216, 0x1F7, 0x225, 0x218, 0x1A7, 0xB6, 0x201, 0x289, 0x1E2, 0x1D9, 0x1A1, 0x242, 0x23E, 0x202, 0x23F, 0x114, 0x13D, 0x101, 0x102, 0xF6, 0x116, 0x114, 0x58, 0xFE5, 0x279, 0x279, 0x305, 0x279, 0x104, 0x8D, 0x12D, 0x2F3, 0x2F3, 0x886, 0x8A, 0x260, 0x53D, 0x105, 0x13C, 0x13C, 0x194, 0xF24, 0x168, 0x168, 0xC48, 0x15A0, 0x395, 0x1724, 0xCFC, 0x2E, 0xB6B, 0x2D3, 0x3FA, 0x3FC, 0x545, 0x545, 0x684, 0x131B, 0x12E3, 0x125, 0x12D, 0x12B, 0x136, 0x1100, 0x1, 0xF11, 0x158, 0x158, 0x34E, 0x112, 0x11D, 0x11D, 0x109, 0x20B, 0x3AC, 0x3E8, 0x289, 0x104, 0x104, 0xF6, 0x105, 0x194, 0x120, 0x10F, 0x120, 0x377, 0xF6, 0x1, 0x175, 0x6F5, 0x41A, 0x2A6, 0x438, 0x159, 0x18B, 0x140, 0xC48, 0x314, 0x51A, 0x1, 0x269, 0x116, 0x116, 0x551, 0x105, 0x105, 0x6E4, 0x434, 0x10A, 0x10A, 0x3AF, 0xDB, 0xDB, 0xDB, 0xDB, 0x738, 0x182, 0x773, 0x104, 0x2AE, 0x105, 0x11B, 0x11E, 0x1D2, 0x119, 0x11F, 0x1A3, 0xDF, 0x119, 0x11C, 0x5DF, 0x57D, 0x134, 0x157, 0x105, 0x120, 0xE7, 0x487, 0xFB8, 0x536, 0x38F, 0xF67, 0x495, 0x515, 0x33A, 0x45A, 0x143, 0x1F0, 0x119, 0x1B3, 0x1C1, 0x1179, 0x1A1, 0x2B4, 0x2B4, 0x346, 0x171, 0x2AD, 0x2AD, 0x1A3, 0x1A3, 0x5C, 0x5C, 0x169, 0x170, 0x2AD, 0x2AD, 0x1A6, 0x1A6, 0x217, 0x77, 0x2D1, 0x30A, 0x2AD, 0x2AD, 0x5C, 0x1AA, 0x1AA, 0x65, 0x60, 0x1A3, 0x1A3, 0xA8, 0x7B, 0xE87, 0x12B, 0x138, 0x38C, 0x10A, 0x10A, 0x12C, 0x12C, 0x54F, 0x10D, 0x111, 0x10D, 0x10D, 0xE7B, 0x4FC, 0x4FE, 0x2D7, 0x68F, 0x18B, 0x18B, 0x1, 0xA22, 0x3BF, 0x5F, 0x5E, 0x206, 0x1, 0xFDF, 0xE09, 0x1368, 0x1040, 0x1186, 0xFBA, 0x1555, 0x28A, 0x1, 0xCCA, 0x10E, 0x4A8, 0x39A, 0xC1A, 0x13C, 0x31C, 0x35F, 0x48C, 0x1F4, 0x5FB, 0x64C, 0xF5, 0x56E, 0x1AA, 0x19C, 0x82A, 0x554, 0x241, 0x22E, 0x6A, 0x143, 0x480, 0x42C, 0x263, 0x2FB, 0x2FB, 0x388, 0x2A4, 0x4EA, 0x4C5, 0x479, 0x515, 0x431, 0x44B, 0x46F, 0x3CC, 0x398, 0x56B, 0x5C8, 0x2B6, 0x3D9, 0x49D, 0x382, 0x2FF, 0x1F5, 0x525, 0xEFF, 0xBE, 0xC4, 0x90, 0x10F, 0x10F, 0x10F, 0x10F, 0x104, 0x104, 0x104, 0x115, 0x115, 0x115, 0x115, 0x3FC, 0x13C, 0x448, 0x733, 0x288, 0x283, 0x127, 0x85D, 0x11CC, 0x6AA, 0x429, 0x125, 0xA47, 0x74, 0x503, 0x62, 0x69, 0x568, 0x10F, 0x104, 0x1257, 0x39D, 0x28D, 0x3AA, 0x101, 0xF4, 0xDF, 0x20F, 0x104, 0x50E, 0x280, 0x280, 0x280, 0x115, 0x2E0, 0xEF8, 0x6FD, 0x7D4, 0x10A1, 0x436, 0x1F8, 0x51B, 0x223, 0xFFD, 0x5B2, 0x308, 0x20A, 0x1CC, 0x1228, 0x487, 0x101, 0x1EC, 0x272, 0x80, 0x900, 0x57D, 0xC4D, 0x421, 0x371, 0x1A7, 0x358, 0x325, 0x112, 0x3FB, 0x5D6, 0x454, 0x8A3, 0xE59, 0x1BE, 0x410, 0x403, 0x3FA, 0x410, 0x3AD, 0x400, 0x2E9, 0x1E4, 0xFBC, 0xC84, 0x655, 0x21A, 0x180, 0x12F6, 0xD0E, 0x1A6, 0x5C, 0x64, 0x159, 0x120, 0x166, 0x19A, 0x101, 0x213, 0xD7, 0x2C9, 0x225, 0x745, 0x705, 0x5D4, 0x9BD, 0x3B2, 0x4B7, 0x2F5, 0x118, 0x389, 0x11A, 0x1D4, 0xFA, 0x150, 0x126, 0x3E9, 0x105, 0x29B, 0x67A, 0x3C1, 0x3C1, 0x3C1, 0x6D8, 0x3C1]
};
const oct13_map = {
    sectors: [0x0, 0x3A, 0x74, 0xAE, 0xF8, 0xF8, 0xF8, 0xF8, 0x260, 0x260, 0x545, 0x6F8, 0x8B0, 0xA5C, 0xC17, 0xDD8, 0xF94, 0x1147, 0x1300, 0x1493, 0x1649, 0x181F, 0x19AD, 0x1B6D, 0x1D03, 0x1EA6, 0x2049, 0x21FE, 0x23A0, 0x254A, 0x26EA, 0x2876, 0x2A01, 0x2B64, 0x2D12, 0x2E8C, 0x3019, 0x31D2, 0x3375, 0x3518, 0x36AB, 0x384F, 0x3A0F, 0x3BBF, 0x3D5F, 0x3F16, 0x40CD, 0x427F, 0x4444, 0x45ED, 0x478A, 0x492A, 0x4AB6, 0x4C5D, 0x4DFE, 0x4F8D, 0x5122, 0x52F0, 0x54C6, 0x567B, 0x57F3, 0x59A4, 0x5B64, 0x5D08, 0x5EC8, 0x6088, 0x6248, 0x63F4, 0x65C7, 0x676A, 0x6908, 0x6AA8, 0x6C7B, 0x6E4E, 0x7019, 0x719D, 0x735E, 0x7511, 0x76C4, 0x786D, 0x7A20, 0x7BD4, 0x7D8B, 0x7F3A, 0x80ED, 0x82AB, 0x843E, 0x85F4, 0x87BA, 0x8948, 0x8B03, 0x8C99, 0x8E3C, 0x8FE0, 0x9195, 0x9336, 0x94DE, 0x967E, 0x980A, 0x9995, 0x9AF8, 0x9CAE, 0x9E28, 0x9FB5, 0xA16E, 0xA30B, 0xA4A8, 0xA63B, 0xA7DF, 0xA99F, 0xAB4F, 0xACEF, 0xAEA4, 0xB059, 0xB20A, 0xB3C0, 0xB569, 0xB706, 0xB8A6, 0xBA32, 0xBBD9, 0xBD79, 0xBF08, 0xC09D, 0xC26B, 0xC42E, 0xC5E3, 0xC75B, 0xC909, 0xCAC6, 0xCC6A, 0xCE2A, 0xCFEA, 0xD1AA, 0xD349, 0xD51C, 0xD6BF, 0xD85D, 0xD9FD, 0xDBD0, 0xDDA3, 0xDF6E, 0xE0F2, 0xE2B3, 0xE466, 0x10BAE, 0x118AD, 0x11CFA, 0x1211F, 0x12544, 0x12905, 0x12E7E, 0x1323F, 0x1368C, 0x13A25, 0x13E36, 0x141E3, 0x14590, 0x14A55, 0x14E3E, 0x1515F, 0x15520, 0x15981, 0x15D92, 0x1627F, 0x1676C, 0x16BCD, 0x17092, 0x17543, 0x17990, 0x17DDD, 0x18216, 0x18663, 0x18A24, 0x18F9D, 0x1935E, 0x1971F, 0x19AE0, 0x19E79, 0x1A23A, 0x1A64B, 0x1ADB8, 0x1B525, 0x1BC92, 0x1BF27, 0x1C1BC, 0x1C929, 0x1D096, 0x1D42F, 0x1D7C8, 0x1DF35, 0x1E256, 0x1E437, 0x1E974, 0x1ED85, 0x1F196, 0x1F43F, 0x1F850, 0x1FD8D, 0x203A6, 0x209BF, 0x20D80, 0x211CD, 0x215F2, 0x21A17, 0x21DD8, 0x22351, 0x22712, 0x22B5F, 0x22FE8, 0x233F9, 0x23882, 0x23D33, 0x241F8, 0x24645, 0x24966, 0x24E17, 0x25278, 0x25689, 0x25AD6, 0x25FC3, 0x26424, 0x268E9, 0x26D9A, 0x271E7, 0x27634, 0x27A6D, 0x27EBA, 0x28343, 0x28754, 0x28BA1, 0x28FEE, 0x2949F, 0x29838, 0x29BF9, 0x2A00A, 0x2A457, 0x2A8A4, 0x2ACF1, 0x2AF86, 0x2B21B, 0x2B668, 0x2BAB5, 0x2BE4E, 0x2C1E7, 0x2C954, 0x2CC75, 0x2D126, 0x2D663, 0x2DA74, 0x2DE85, 0x2E12E, 0x2E53F, 0x2EA7C, 0x2F095, 0x2F6AE, 0x2FB37, 0x2FB5E, 0x2FB86, 0x2FB9C, 0x2FBB6, 0x2FBDD, 0x2FC12, 0x2FC31, 0x2FC59, 0x2FC79, 0x2FC8E, 0x2FCAE, 0x2FCD6, 0x2FCFC, 0x2FD1B, 0x2FD37, 0x2FD62, 0x2FD79, 0x2FD91, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FDA2, 0x2FE13, 0x2FE27, 0x2FE3E, 0x2FE73, 0x2FEA2, 0x2FEC5, 0x2FF02, 0x2FF23, 0x2FF58, 0x2FF5A, 0x2FF5C, 0x2FF5E, 0x2FF60, 0x2FF96, 0x2FFC8, 0x2FFF9, 0x30024, 0x3005D, 0x300BB, 0x3010F, 0x30173, 0x301AA, 0x301E0, 0x30217, 0x3024F, 0x30286, 0x302B6, 0x302E6, 0x3031C, 0x30350, 0x3037C, 0x303B5, 0x303E4, 0x30417, 0x30419, 0x3044C, 0x3047E, 0x304E3, 0x304E5, 0x30519, 0x30587, 0x305DC, 0x30627, 0x30629, 0x306EB, 0x3071E, 0x30751, 0x307A6, 0x307B5, 0x307D7, 0x307DE, 0x307F4, 0x30830, 0x3086C, 0x308A8, 0x308B7, 0x308D7, 0x30905, 0x3093D, 0x30954, 0x30976, 0x30982, 0x3098F, 0x3099B, 0x309C2, 0x309F2, 0x30A00, 0x30A2C, 0x30A4E, 0x30A7F, 0x30A84, 0x30A99, 0x30AB1, 0x30AB6, 0x30ACB, 0x30AE3, 0x30AF6, 0x30B29, 0x30B4C, 0x30B63, 0x30B76, 0x30BA7, 0x30BB6, 0x30BBD, 0x30BEE, 0x30C21, 0x30C4E, 0x30C80, 0x30C9F, 0x30CAF, 0x30CB5, 0x30CE5, 0x30D17, 0x30D48, 0x30D60, 0x30D91, 0x30DBB, 0x30DE9, 0x30DF6, 0x30E0E, 0x30E2C, 0x30E4A, 0x30E78, 0x30E7D, 0x30E99, 0x30EAE, 0x30ECA, 0x30EE6, 0x30EEB, 0x30EEF, 0x30EF3, 0x30EF8, 0x30EFE, 0x30F02, 0x30F18, 0x30F2B, 0x30F47, 0x30F67, 0x30F8A, 0x30FAF, 0x30FCE, 0x30FF2, 0x31010, 0x3102D, 0x3104B, 0x31069, 0x31087, 0x310A5, 0x310C3, 0x310E6, 0x31109, 0x3112C, 0x3114D, 0x31173, 0x31199, 0x311BC, 0x311E0, 0x31204, 0x31230, 0x3124C, 0x31264, 0x3127C, 0x3129D, 0x312B8, 0x312D1, 0x312D3, 0x314C2, 0x314C3, 0x3151F, 0x31520, 0x3157C, 0x3157D, 0x31740, 0x31741, 0x317D3, 0x317D4, 0x3184F, 0x318F0, 0x3199C, 0x31A44, 0x31AE8, 0x31B9A, 0x31C49, 0x31D09, 0x31DAD, 0x31E7D, 0x31F56, 0x3202B, 0x32103, 0x321E6, 0x322BE, 0x3239D, 0x3243D, 0x3243E, 0x3243F, 0x32440, 0x32441, 0x32442, 0x32443, 0x3259E, 0x3259F, 0x325E9, 0x325EF, 0x32692, 0x3269F, 0x327F3, 0x3282A, 0x328DB, 0x329C6, 0x32A82, 0x32B12, 0x32B6D, 0x32BD3, 0x32C61, 0x32D3F, 0x32E1D, 0x32E7D, 0x32E83, 0x32E8A, 0x32E92, 0x32EA4, 0x32EB2, 0x32EBE, 0x32ECA, 0x32ED0, 0x32ED7, 0x32EE8, 0x32EFE, 0x32F07, 0x32F16, 0x32F25, 0x32F3D, 0x32F48, 0x32F53, 0x32F62, 0x32F78, 0x32F87, 0x32F96, 0x32FA1, 0x32FB3, 0x32FC5, 0x32FCC, 0x32FDA, 0x32FE9, 0x32FF6, 0x32FFE, 0x3300E, 0x3301E, 0x33028, 0x34A48, 0x34AC9, 0x34B4A, 0x34BCB, 0x34C4C, 0x34CCD, 0x34D4E, 0x34DCF, 0x34E50, 0x34ED1, 0x34F52, 0x34FD3, 0x35054, 0x350D5, 0x35156, 0x351D7, 0x35258, 0x352D9, 0x3531A, 0x3539B, 0x3541C, 0x3549D, 0x3551E, 0x3559F, 0x35620, 0x356A1, 0x35722, 0x357A3, 0x35824, 0x358A5, 0x35926, 0x359A7, 0x35A28, 0x35AA9, 0x35B2A, 0x35BAB, 0x35C2C, 0x35CAD, 0x35D2E, 0x35DAF, 0x35E30, 0x35EB1, 0x35F32, 0x35FB3, 0x36034, 0x360B5, 0x36136, 0x361B7, 0x36238, 0x362B9, 0x3633A, 0x363BB, 0x3643C, 0x364BD, 0x3653E, 0x365BF, 0x36640, 0x366C1, 0x36742, 0x367C3, 0x36844, 0x368C5, 0x36946, 0x369C7, 0x36A48, 0x36AC9, 0x36B4A, 0x36BCB, 0x36C4C, 0x36CCD, 0x36D4E, 0x36DCF, 0x36E50, 0x36ED1, 0x36F52, 0x36FD3, 0x37054, 0x370D5, 0x37156, 0x371D7, 0x37258, 0x372D9, 0x3735A, 0x373DB, 0x3745C, 0x374DD, 0x3755E, 0x375DF, 0x37660, 0x376E1, 0x37762, 0x377E3, 0x37864, 0x378E5, 0x37966, 0x379E7, 0x37A68, 0x37AE9, 0x37B6A, 0x37BEB, 0x37C6C, 0x37CED, 0x38115, 0x3859D, 0x38905, 0x38BC9, 0x38E34, 0x38F7D, 0x3975A, 0x39F9B, 0x3A3C3, 0x3A84B, 0x3ABB3, 0x3AE77, 0x3B0E2, 0x3B50A, 0x3B992, 0x3BCFA, 0x3BFBE, 0x3C229, 0x3CA6C, 0x3CAEA, 0x3CB68, 0x3D365, 0x3D523, 0x3D678, 0x3DC65, 0x3E191, 0x3E7F6, 0x3ECB4, 0x3F457, 0x3F55C, 0x3F5E4, 0x3F66C, 0x3F787, 0x3F8A2, 0x3F92A, 0x3FDFD, 0x404F1, 0x4069C, 0x40842, 0x40AA8, 0x40DE9, 0x40ECD, 0x40F88, 0x41402, 0x41C88, 0x41CF1, 0x420B7, 0x4247D, 0x429EB, 0x42AC5, 0x42BD1, 0x42CF6, 0x42D9B, 0x43406, 0x43B3A, 0x43CC6, 0x43DEA, 0x44F20, 0x4516F, 0x453C1, 0x456AF, 0x4599D, 0x45AD7, 0x45C13, 0x4609E, 0x462E3, 0x4646F, 0x46914, 0x46B1F, 0x46C2B, 0x47082, 0x4767C, 0x47BDF, 0x47CA1, 0x4815A, 0x48C63, 0x4901D, 0x4925A, 0x4944D, 0x4A4AF, 0x4B212, 0x4B275, 0x4B473, 0x4B671, 0x4B87C, 0x4BACE, 0x4BCE1, 0x4BE88, 0x4C095, 0x4C2A2, 0x4C438, 0x4C64F, 0x4C75C, 0x4C895, 0x4DB87, 0x4DCC3, 0x4DF9A, 0x4E083, 0x4E5AC, 0x4F1A2, 0x4F387, 0x4F56C, 0x4F837, 0x4F920, 0x4FE35, 0x502C8, 0x5051E, 0x50661, 0x50A18, 0x50DA3, 0x5123A, 0x517A4, 0x52687, 0x52A2C, 0x52F93, 0x5361B, 0x538A3, 0x539EF, 0x5452F, 0x545B4, 0x5467B, 0x54888, 0x54FB2, 0x5532B, 0x553AC, 0x55434, 0x5579D, 0x5598D, 0x55B83, 0x55D75, 0x55F08, 0x56131, 0x56249, 0x56361, 0x56479, 0x56591, 0x566A9, 0x566AA, 0x56B3D, 0x56C55, 0x56D6D, 0x56E85, 0x56F9D, 0x570B5, 0x583AE, 0x5883D, 0x58948, 0x58B6A, 0x58FF5, 0x5936E, 0x59487, 0x595A0, 0x59795, 0x59BAB, 0x59C34, 0x59CBD, 0x5A039, 0x5A1DB, 0x5A37D, 0x5A792, 0x5A982, 0x5AB72, 0x5AD62, 0x5AF52, 0x5BA43, 0x5C263, 0x5C8BF, 0x5CE5C, 0x5D3E1, 0x5D4C8, 0x5DAD4, 0x5E130, 0x5E1E6, 0x5E8B5, 0x5E96F, 0x5EA29, 0x5EAE3, 0x5EF15, 0x5F3A9, 0x5F826, 0x5FCD1, 0x5FFFC, 0x5FFFD, 0x60466, 0x6060C, 0x6091A, 0x61B90, 0x61DFC, 0x62140, 0x62252, 0x62356, 0x6254C, 0x6265E, 0x62790, 0x62884, 0x62990, 0x62D37, 0x62EB4, 0x63041, 0x63153, 0x63285, 0x63A44, 0x6465B, 0x64CF0, 0x64F46, 0x65043, 0x65140, 0x65293, 0x65CA9, 0x66210, 0x6653E, 0x66807, 0x6688F, 0x669E4, 0x66C84, 0x67D61, 0x67DEF, 0x67FAB, 0x68158, 0x68273, 0x68543, 0x6862A, 0x68711, 0x689D1, 0x68AD6, 0x68B53, 0x68BD0, 0x68BD1, 0x69619, 0x6961A, 0x69698, 0x6A331, 0x6A3F2, 0x6A4B1, 0x6A514, 0x6A9CA, 0x6AAE5, 0x6AC00, 0x6AD1B, 0x6AE36, 0x6AF51, 0x6B06C, 0x6B166, 0x6B260, 0x6B35A, 0x6B454, 0x6B54E, 0x6B648, 0x6B72F, 0x6B816, 0x6B8FD, 0x6B9E4, 0x6BACB, 0x6BBB2, 0x6BC99, 0x6BD80, 0x6CCBA, 0x6CEAE, 0x6D09E, 0x6D17B, 0x6D258, 0x6D2E3, 0x6D3A9, 0x6D552, 0x6D95D, 0x6DC3B, 0x6DD51, 0x6DE63, 0x6E0E5, 0x6E7F0, 0x6E8E3, 0x6EBA3, 0x6EC99, 0x6F0BD, 0x6F3B8, 0x6F6B3, 0x6F9AE, 0x6FB35, 0x70ED7, 0x70F7F, 0x710A6, 0x711CC, 0x712B2, 0x713CF, 0x714D3, 0x715D7, 0x71857, 0x71975, 0x71C8C, 0x71EA6, 0x72088, 0x72365, 0x73707, 0x737AF, 0x738D6, 0x739FC, 0x73AE2, 0x73BFF, 0x73D03, 0x73E07, 0x74087, 0x741A5, 0x744BC, 0x746D6, 0x748B8, 0x74B95, 0x74FB4, 0x7613D, 0x77210, 0x77645, 0x77767, 0x77D30, 0x78C6A, 0x79E78, 0x79F6D, 0x7A0AA, 0x7A165, 0x7A29E, 0x7A5A2, 0x7A8D2, 0x7AADF, 0x7ADDC, 0x7B0AD, 0x7B18C, 0x7B38A, 0x7BB8F, 0x7BBC0, 0x7BFCE, 0x7C107, 0x7C240, 0x7C5FD, 0x7C70D, 0x7C81D, 0x7CDB1, 0x7CE77, 0x7CF3D, 0x7D2EA, 0x7D4AB, 0x7D66C, 0x7DCB0, 0x7E053, 0x7E3F6, 0x7E8DC, 0x7E9E0, 0x7EAE4, 0x7FD04, 0x7FDE1, 0x7FECE, 0x80028, 0x801DF, 0x8037E, 0x804CD, 0x8076B, 0x8094D, 0x80B61, 0x80D77, 0x80F6E, 0x81193, 0x813AB, 0x81552, 0x81608, 0x81809, 0x81A92, 0x81C74, 0x81E4D, 0x81FEE, 0x82230, 0x8246E, 0x82670, 0x828AF, 0x829C3, 0x82B00, 0x82C01, 0x82D03, 0x82DF9, 0x82F0F, 0x83023, 0x8307B, 0x84060, 0x842D9, 0x84552, 0x84857, 0x84AD0, 0x84BD4, 0x84C61, 0x84D8E, 0x85081, 0x85374, 0x85BFA, 0x85C84, 0x85EE4, 0x86421, 0x86526, 0x86662, 0x8679E, 0x86932, 0x87856, 0x879BE, 0x87B26, 0x8876E, 0x89D0E, 0x8A0A3, 0x8B7C7, 0x8C4C3, 0x8C4F1, 0x8D05C, 0x8D32F, 0x8D729, 0x8DB25, 0x8E06A, 0x8E5AF, 0x8EC33, 0x8FF4E, 0x91231, 0x91356, 0x91483, 0x915AE, 0x916E4, 0x927E4, 0x927E5, 0x936F6, 0x9384E, 0x939A6, 0x93CF4, 0x93E06, 0x93F23, 0x94040, 0x94149, 0x94354, 0x94700, 0x94AE8, 0x94D71, 0x94E75, 0x94F79, 0x9506F, 0x95174, 0x95308, 0x95428, 0x95537, 0x95657, 0x959CE, 0x95AC4, 0x95AC5, 0x95C3A, 0x9632F, 0x96749, 0x969EF, 0x96E27, 0x96F80, 0x9710B, 0x9724B, 0x97E93, 0x981A7, 0x986C1, 0x986C2, 0x9892B, 0x98A41, 0x98B57, 0x990A8, 0x991AD, 0x992B2, 0x99996, 0x99DCA, 0x99ED4, 0x99FDE, 0x9A38D, 0x9A468, 0x9A543, 0x9A61E, 0x9A6F9, 0x9AE31, 0x9AFB3, 0x9B726, 0x9B82A, 0x9BAD8, 0x9BBDD, 0x9BCF8, 0x9BE16, 0x9BFE8, 0x9C101, 0x9C220, 0x9C3C3, 0x9C4A2, 0x9C5BB, 0x9C6D7, 0x9CCB6, 0x9D233, 0x9D367, 0x9D4BE, 0x9D5C3, 0x9D6E3, 0x9D7CA, 0x9DC51, 0x9EC09, 0x9F13F, 0x9F4CE, 0xA0435, 0xA08CA, 0xA0DDF, 0xA1119, 0xA1573, 0xA16B6, 0xA18A6, 0xA19BF, 0xA1B72, 0xA1D33, 0xA2EAC, 0xA304D, 0xA3301, 0xA35B5, 0xA38FB, 0xA3A6C, 0xA3D19, 0xA3FC6, 0xA4169, 0xA430C, 0xA4368, 0xA43C4, 0xA452D, 0xA469D, 0xA494A, 0xA4BF7, 0xA4D9D, 0xA4F43, 0xA515A, 0xA51D1, 0xA54A2, 0xA57AC, 0xA5A59, 0xA5D06, 0xA5D62, 0xA5F0C, 0xA60B6, 0xA611B, 0xA617B, 0xA631E, 0xA64C1, 0xA6569, 0xA65E4, 0xA746B, 0xA7596, 0xA76CE, 0xA7A5A, 0xA7B64, 0xA7C6E, 0xA7D9A, 0xA7EC6, 0xA8415, 0xA8522, 0xA8633, 0xA8740, 0xA884D, 0xA96C8, 0xA9BC4, 0xAA0C2, 0xAA399, 0xAAA28, 0xAABB3, 0xAAD3E, 0xAAD3F, 0xAB761, 0xABB20, 0xABB7F, 0xABBDD, 0xABDE3, 0xABDE4, 0xACDC3, 0xADBCC, 0xAEF34, 0xAFF74, 0xB10FA, 0xB20B4, 0xB3609, 0xB3893, 0xB3894, 0xB455E, 0xB466C, 0xB4B14, 0xB4EAE, 0xB5AC8, 0xB5C04, 0xB5F20, 0xB627F, 0xB670B, 0xB68FF, 0xB6EFA, 0xB7546, 0xB763B, 0xB7BA9, 0xB7D53, 0xB7EEF, 0xB8719, 0xB8C6D, 0xB8EAE, 0xB90DC, 0xB9146, 0xB9289, 0xB9709, 0xB9B35, 0xB9D98, 0xBA093, 0xBA38E, 0xBA716, 0xBA9BA, 0xBAEA4, 0xBB369, 0xBB7E2, 0xBBCF7, 0xBC128, 0xBC573, 0xBC9E2, 0xBCDAE, 0xBD146, 0xBD6B1, 0xBDC79, 0xBDF2F, 0xBE308, 0xBE7A5, 0xBEB27, 0xBEE26, 0xBF01B, 0xBF540, 0xC043F, 0xC04FD, 0xC05C1, 0xC0651, 0xC0760, 0xC086F, 0xC097E, 0xC0A8D, 0xC0B91, 0xC0C95, 0xC0D99, 0xC0EAE, 0xC0FC3, 0xC10D8, 0xC11ED, 0xC15E9, 0xC1725, 0xC1B6D, 0xC22A0, 0xC2528, 0xC27AB, 0xC28D2, 0xC312F, 0xC42FB, 0xC49A5, 0xC4DCE, 0xC4EF3, 0xC593A, 0xC59AE, 0xC5EB1, 0xC5F13, 0xC5F7C, 0xC64E4, 0xC65F3, 0xC66F7, 0xC794E, 0xC7CEB, 0xC7F78, 0xC8322, 0xC8423, 0xC8517, 0xC85F6, 0xC8805, 0xC8909, 0xC8E17, 0xC9097, 0xC9317, 0xC9597, 0xC96AC, 0xC998C, 0xCA884, 0xCAF81, 0xCB755, 0xCC7F6, 0xCCC2C, 0xCCE24, 0xCD33F, 0xCD562, 0xCE55F, 0xCEB11, 0xCEE19, 0xCF023, 0xCF1EF, 0xD0417, 0xD089E, 0xD099F, 0xD0B8B, 0xD0DFD, 0xD0E7D, 0xD177D, 0xD1CFA, 0xD2947, 0xD2D68, 0xD30D9, 0xD3280, 0xD35D8, 0xD38FD, 0xD3A0F, 0xD3E0A, 0xD43E0, 0xD4834, 0xD50D7, 0xD5F30, 0xD60EE, 0xD64FE, 0xD6901, 0xD6CFB, 0xD710B, 0xD74B8, 0xD78B8, 0xD7BA1, 0xD7D85, 0xD8D41, 0xD99C5, 0xDA01A, 0xDA234, 0xDA3B4, 0xDB6AA, 0xDC3B8, 0xDC55E, 0xDC5BA, 0xDC61E, 0xDC777, 0xDC897, 0xDC9FD, 0xDCB97, 0xDCC98, 0xDCEAB, 0xDCF82, 0xDD24B, 0xDD470, 0xDDBB5, 0xDE2BA, 0xDE88E, 0xDF24B, 0xDF5FD, 0xDFAB4, 0xDFDA9, 0xDFEC1, 0xE024A, 0xE0364, 0xE0538, 0xE0632, 0xE0782, 0xE08A8, 0xE0C91, 0xE0D96, 0xE1031, 0xE16AB, 0xE1A6C, 0xE1E2D, 0xE21EE, 0xE28C6],
    sizes: [0x3A, 0x3A, 0x3A, 0x4A, 0x0, 0x0, 0x0, 0x168, 0x0, 0x2E5, 0x1B3, 0x1B8, 0x1AC, 0x1BB, 0x1C1, 0x1BC, 0x1B3, 0x1B9, 0x193, 0x1B6, 0x1D6, 0x18E, 0x1C0, 0x196, 0x1A3, 0x1A3, 0x1B5, 0x1A2, 0x1AA, 0x1A0, 0x18C, 0x18B, 0x163, 0x1AE, 0x17A, 0x18D, 0x1B9, 0x1A3, 0x1A3, 0x193, 0x1A4, 0x1C0, 0x1B0, 0x1A0, 0x1B7, 0x1B7, 0x1B2, 0x1C5, 0x1A9, 0x19D, 0x1A0, 0x18C, 0x1A7, 0x1A1, 0x18F, 0x195, 0x1CE, 0x1D6, 0x1B5, 0x178, 0x1B1, 0x1C0, 0x1A4, 0x1C0, 0x1C0, 0x1C0, 0x1AC, 0x1D3, 0x1A3, 0x19E, 0x1A0, 0x1D3, 0x1D3, 0x1CB, 0x184, 0x1C1, 0x1B3, 0x1B3, 0x1A9, 0x1B3, 0x1B4, 0x1B7, 0x1AF, 0x1B3, 0x1BE, 0x193, 0x1B6, 0x1C6, 0x18E, 0x1BB, 0x196, 0x1A3, 0x1A4, 0x1B5, 0x1A1, 0x1A8, 0x1A0, 0x18C, 0x18B, 0x163, 0x1B6, 0x17A, 0x18D, 0x1B9, 0x19D, 0x19D, 0x193, 0x1A4, 0x1C0, 0x1B0, 0x1A0, 0x1B5, 0x1B5, 0x1B1, 0x1B6, 0x1A9, 0x19D, 0x1A0, 0x18C, 0x1A7, 0x1A0, 0x18F, 0x195, 0x1CE, 0x1C3, 0x1B5, 0x178, 0x1AE, 0x1BD, 0x1A4, 0x1C0, 0x1C0, 0x1C0, 0x19F, 0x1D3, 0x1A3, 0x19E, 0x1A0, 0x1D3, 0x1D3, 0x1CB, 0x184, 0x1C1, 0x1B3, 0x2748, 0xCFF, 0x44D, 0x425, 0x425, 0x3C1, 0x579, 0x3C1, 0x44D, 0x399, 0x411, 0x3AD, 0x3AD, 0x4C5, 0x3E9, 0x321, 0x3C1, 0x461, 0x411, 0x4ED, 0x4ED, 0x461, 0x4C5, 0x4B1, 0x44D, 0x44D, 0x439, 0x44D, 0x3C1, 0x579, 0x3C1, 0x3C1, 0x3C1, 0x399, 0x3C1, 0x411, 0x76D, 0x76D, 0x76D, 0x295, 0x295, 0x76D, 0x76D, 0x399, 0x399, 0x76D, 0x321, 0x1E1, 0x53D, 0x411, 0x411, 0x2A9, 0x411, 0x53D, 0x619, 0x619, 0x3C1, 0x44D, 0x425, 0x425, 0x3C1, 0x579, 0x3C1, 0x44D, 0x489, 0x411, 0x489, 0x4B1, 0x4C5, 0x44D, 0x321, 0x4B1, 0x461, 0x411, 0x44D, 0x4ED, 0x461, 0x4C5, 0x4B1, 0x44D, 0x44D, 0x439, 0x44D, 0x489, 0x411, 0x44D, 0x44D, 0x4B1, 0x399, 0x3C1, 0x411, 0x44D, 0x44D, 0x44D, 0x295, 0x295, 0x44D, 0x44D, 0x399, 0x399, 0x76D, 0x321, 0x4B1, 0x53D, 0x411, 0x411, 0x2A9, 0x411, 0x53D, 0x619, 0x619, 0x489, 0x27, 0x28, 0x16, 0x1A, 0x27, 0x35, 0x1F, 0x28, 0x20, 0x15, 0x20, 0x28, 0x26, 0x1F, 0x1C, 0x2B, 0x17, 0x18, 0x11, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x71, 0x14, 0x17, 0x35, 0x2F, 0x23, 0x3D, 0x21, 0x35, 0x2, 0x2, 0x2, 0x2, 0x36, 0x32, 0x31, 0x2B, 0x39, 0x5E, 0x54, 0x64, 0x37, 0x36, 0x37, 0x38, 0x37, 0x30, 0x30, 0x36, 0x34, 0x2C, 0x39, 0x2F, 0x33, 0x2, 0x33, 0x32, 0x65, 0x2, 0x34, 0x6E, 0x55, 0x4B, 0x2, 0xC2, 0x33, 0x33, 0x55, 0xF, 0x22, 0x7, 0x16, 0x3C, 0x3C, 0x3C, 0xF, 0x20, 0x2E, 0x38, 0x17, 0x22, 0xC, 0xD, 0xC, 0x27, 0x30, 0xE, 0x2C, 0x22, 0x31, 0x5, 0x15, 0x18, 0x5, 0x15, 0x18, 0x13, 0x33, 0x23, 0x17, 0x13, 0x31, 0xF, 0x7, 0x31, 0x33, 0x2D, 0x32, 0x1F, 0x10, 0x6, 0x30, 0x32, 0x31, 0x18, 0x31, 0x2A, 0x2E, 0xD, 0x18, 0x1E, 0x1E, 0x2E, 0x5, 0x1C, 0x15, 0x1C, 0x1C, 0x5, 0x4, 0x4, 0x5, 0x6, 0x4, 0x16, 0x13, 0x1C, 0x20, 0x23, 0x25, 0x1F, 0x24, 0x1E, 0x1D, 0x1E, 0x1E, 0x1E, 0x1E, 0x1E, 0x23, 0x23, 0x23, 0x21, 0x26, 0x26, 0x23, 0x24, 0x24, 0x2C, 0x1C, 0x18, 0x18, 0x21, 0x1B, 0x19, 0x2, 0x1EF, 0x1, 0x5C, 0x1, 0x5C, 0x1, 0x1C3, 0x1, 0x92, 0x1, 0x7B, 0xA1, 0xAC, 0xA8, 0xA4, 0xB2, 0xAF, 0xC0, 0xA4, 0xD0, 0xD9, 0xD5, 0xD8, 0xE3, 0xD8, 0xDF, 0xA0, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x15B, 0x1, 0x4A, 0x6, 0xA3, 0xD, 0x154, 0x37, 0xB1, 0xEB, 0xBC, 0x90, 0x5B, 0x66, 0x8E, 0xDE, 0xDE, 0x60, 0x6, 0x7, 0x8, 0x12, 0xE, 0xC, 0xC, 0x6, 0x7, 0x11, 0x16, 0x9, 0xF, 0xF, 0x18, 0xB, 0xB, 0xF, 0x16, 0xF, 0xF, 0xB, 0x12, 0x12, 0x7, 0xE, 0xF, 0xD, 0x8, 0x10, 0x10, 0xA, 0x1A20, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x41, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x428, 0x488, 0x368, 0x2C4, 0x26B, 0x149, 0x7DD, 0x841, 0x428, 0x488, 0x368, 0x2C4, 0x26B, 0x428, 0x488, 0x368, 0x2C4, 0x26B, 0x843, 0x7E, 0x7E, 0x7FD, 0x1BE, 0x155, 0x5ED, 0x52C, 0x665, 0x4BE, 0x7A3, 0x105, 0x88, 0x88, 0x11B, 0x11B, 0x88, 0x4D3, 0x6F4, 0x1AB, 0x1A6, 0x266, 0x341, 0xE4, 0xBB, 0x47A, 0x886, 0x69, 0x3C6, 0x3C6, 0x56E, 0xDA, 0x10C, 0x125, 0xA5, 0x66B, 0x734, 0x18C, 0x124, 0x1136, 0x24F, 0x252, 0x2EE, 0x2EE, 0x13A, 0x13C, 0x48B, 0x245, 0x18C, 0x4A5, 0x20B, 0x10C, 0x457, 0x5FA, 0x563, 0xC2, 0x4B9, 0xB09, 0x3BA, 0x23D, 0x1F3, 0x1062, 0xD63, 0x63, 0x1FE, 0x1FE, 0x20B, 0x252, 0x213, 0x1A7, 0x20D, 0x20D, 0x196, 0x217, 0x10D, 0x139, 0x12F2, 0x13C, 0x2D7, 0xE9, 0x529, 0xBF6, 0x1E5, 0x1E5, 0x2CB, 0xE9, 0x515, 0x493, 0x256, 0x143, 0x3B7, 0x38B, 0x497, 0x56A, 0xEE3, 0x3A5, 0x567, 0x688, 0x288, 0x14C, 0xB40, 0x85, 0xC7, 0x20D, 0x72A, 0x379, 0x81, 0x88, 0x369, 0x1F0, 0x1F6, 0x1F2, 0x193, 0x229, 0x118, 0x118, 0x118, 0x118, 0x118, 0x1, 0x493, 0x118, 0x118, 0x118, 0x118, 0x118, 0x12F9, 0x48F, 0x10B, 0x222, 0x48B, 0x379, 0x119, 0x119, 0x1F5, 0x416, 0x89, 0x89, 0x37C, 0x1A2, 0x1A2, 0x415, 0x1F0, 0x1F0, 0x1F0, 0x1F0, 0xAF1, 0x820, 0x65C, 0x59D, 0x585, 0xE7, 0x60C, 0x65C, 0xB6, 0x6CF, 0xBA, 0xBA, 0xBA, 0x432, 0x494, 0x47D, 0x4AB, 0x32B, 0x1, 0x469, 0x1A6, 0x30E, 0x1276, 0x26C, 0x344, 0x112, 0x104, 0x1F6, 0x112, 0x132, 0xF4, 0x10C, 0x3A7, 0x17D, 0x18D, 0x112, 0x132, 0x7BF, 0xC17, 0x695, 0x256, 0xFD, 0xFD, 0x153, 0xA16, 0x567, 0x32E, 0x2C9, 0x88, 0x155, 0x2A0, 0x10DD, 0x8E, 0x1BC, 0x1AD, 0x11B, 0x2D0, 0xE7, 0xE7, 0x2C0, 0x105, 0x7D, 0x7D, 0x1, 0xA48, 0x1, 0x7E, 0xC99, 0xC1, 0xBF, 0x63, 0x4B6, 0x11B, 0x11B, 0x11B, 0x11B, 0x11B, 0x11B, 0xFA, 0xFA, 0xFA, 0xFA, 0xFA, 0xFA, 0xE7, 0xE7, 0xE7, 0xE7, 0xE7, 0xE7, 0xE7, 0xE7, 0xF3A, 0x1F4, 0x1F0, 0xDD, 0xDD, 0x8B, 0xC6, 0x1A9, 0x40B, 0x2DE, 0x116, 0x112, 0x282, 0x70B, 0xF3, 0x2C0, 0xF6, 0x424, 0x2FB, 0x2FB, 0x2FB, 0x187, 0x13A2, 0xA8, 0x127, 0x126, 0xE6, 0x11D, 0x104, 0x104, 0x280, 0x11E, 0x317, 0x21A, 0x1E2, 0x2DD, 0x13A2, 0xA8, 0x127, 0x126, 0xE6, 0x11D, 0x104, 0x104, 0x280, 0x11E, 0x317, 0x21A, 0x1E2, 0x2DD, 0x41F, 0x1189, 0x10D3, 0x435, 0x122, 0x5C9, 0xF3A, 0x120E, 0xF5, 0x13D, 0xBB, 0x139, 0x304, 0x330, 0x20D, 0x2FD, 0x2D1, 0xDF, 0x1FE, 0x805, 0x31, 0x40E, 0x139, 0x139, 0x3BD, 0x110, 0x110, 0x594, 0xC6, 0xC6, 0x3AD, 0x1C1, 0x1C1, 0x644, 0x3A3, 0x3A3, 0x4E6, 0x104, 0x104, 0x1220, 0xDD, 0xED, 0x15A, 0x1B7, 0x19F, 0x14F, 0x29E, 0x1E2, 0x214, 0x216, 0x1F7, 0x225, 0x218, 0x1A7, 0xB6, 0x201, 0x289, 0x1E2, 0x1D9, 0x1A1, 0x242, 0x23E, 0x202, 0x23F, 0x114, 0x13D, 0x101, 0x102, 0xF6, 0x116, 0x114, 0x58, 0xFE5, 0x279, 0x279, 0x305, 0x279, 0x104, 0x8D, 0x12D, 0x2F3, 0x2F3, 0x886, 0x8A, 0x260, 0x53D, 0x105, 0x13C, 0x13C, 0x194, 0xF24, 0x168, 0x168, 0xC48, 0x15A0, 0x395, 0x1724, 0xCFC, 0x2E, 0xB6B, 0x2D3, 0x3FA, 0x3FC, 0x545, 0x545, 0x684, 0x131B, 0x12E3, 0x125, 0x12D, 0x12B, 0x136, 0x1100, 0x1, 0xF11, 0x158, 0x158, 0x34E, 0x112, 0x11D, 0x11D, 0x109, 0x20B, 0x3AC, 0x3E8, 0x289, 0x104, 0x104, 0xF6, 0x105, 0x194, 0x120, 0x10F, 0x120, 0x377, 0xF6, 0x1, 0x175, 0x6F5, 0x41A, 0x2A6, 0x438, 0x159, 0x18B, 0x140, 0xC48, 0x314, 0x51A, 0x1, 0x269, 0x116, 0x116, 0x551, 0x105, 0x105, 0x6E4, 0x434, 0x10A, 0x10A, 0x3AF, 0xDB, 0xDB, 0xDB, 0xDB, 0x738, 0x182, 0x773, 0x104, 0x2AE, 0x105, 0x11B, 0x11E, 0x1D2, 0x119, 0x11F, 0x1A3, 0xDF, 0x119, 0x11C, 0x5DF, 0x57D, 0x134, 0x157, 0x105, 0x120, 0xE7, 0x487, 0xFB8, 0x536, 0x38F, 0xF67, 0x495, 0x515, 0x33A, 0x45A, 0x143, 0x1F0, 0x119, 0x1B3, 0x1C1, 0x1179, 0x1A1, 0x2B4, 0x2B4, 0x346, 0x171, 0x2AD, 0x2AD, 0x1A3, 0x1A3, 0x5C, 0x5C, 0x169, 0x170, 0x2AD, 0x2AD, 0x1A6, 0x1A6, 0x217, 0x77, 0x2D1, 0x30A, 0x2AD, 0x2AD, 0x5C, 0x1AA, 0x1AA, 0x65, 0x60, 0x1A3, 0x1A3, 0xA8, 0x7B, 0xE87, 0x12B, 0x138, 0x38C, 0x10A, 0x10A, 0x12C, 0x12C, 0x54F, 0x10D, 0x111, 0x10D, 0x10D, 0xE7B, 0x4FC, 0x4FE, 0x2D7, 0x68F, 0x18B, 0x18B, 0x1, 0xA22, 0x3BF, 0x5F, 0x5E, 0x206, 0x1, 0xFDF, 0xE09, 0x1368, 0x1040, 0x1186, 0xFBA, 0x1555, 0x28A, 0x1, 0xCCA, 0x10E, 0x4A8, 0x39A, 0xC1A, 0x13C, 0x31C, 0x35F, 0x48C, 0x1F4, 0x5FB, 0x64C, 0xF5, 0x56E, 0x1AA, 0x19C, 0x82A, 0x554, 0x241, 0x22E, 0x6A, 0x143, 0x480, 0x42C, 0x263, 0x2FB, 0x2FB, 0x388, 0x2A4, 0x4EA, 0x4C5, 0x479, 0x515, 0x431, 0x44B, 0x46F, 0x3CC, 0x398, 0x56B, 0x5C8, 0x2B6, 0x3D9, 0x49D, 0x382, 0x2FF, 0x1F5, 0x525, 0xEFF, 0xBE, 0xC4, 0x90, 0x10F, 0x10F, 0x10F, 0x10F, 0x104, 0x104, 0x104, 0x115, 0x115, 0x115, 0x115, 0x3FC, 0x13C, 0x448, 0x733, 0x288, 0x283, 0x127, 0x85D, 0x11CC, 0x6AA, 0x429, 0x125, 0xA47, 0x74, 0x503, 0x62, 0x69, 0x568, 0x10F, 0x104, 0x1257, 0x39D, 0x28D, 0x3AA, 0x101, 0xF4, 0xDF, 0x20F, 0x104, 0x50E, 0x280, 0x280, 0x280, 0x115, 0x2E0, 0xEF8, 0x6FD, 0x7D4, 0x10A1, 0x436, 0x1F8, 0x51B, 0x223, 0xFFD, 0x5B2, 0x308, 0x20A, 0x1CC, 0x1228, 0x487, 0x101, 0x1EC, 0x272, 0x80, 0x900, 0x57D, 0xC4D, 0x421, 0x371, 0x1A7, 0x358, 0x325, 0x112, 0x3FB, 0x5D6, 0x454, 0x8A3, 0xE59, 0x1BE, 0x410, 0x403, 0x3FA, 0x410, 0x3AD, 0x400, 0x2E9, 0x1E4, 0xFBC, 0xC84, 0x655, 0x21A, 0x180, 0x12F6, 0xD0E, 0x1A6, 0x5C, 0x64, 0x159, 0x120, 0x166, 0x19A, 0x101, 0x213, 0xD7, 0x2C9, 0x225, 0x745, 0x705, 0x5D4, 0x9BD, 0x3B2, 0x4B7, 0x2F5, 0x118, 0x389, 0x11A, 0x1D4, 0xFA, 0x150, 0x126, 0x3E9, 0x105, 0x29B, 0x67A, 0x3C1, 0x3C1, 0x3C1, 0x6D8, 0x3C1]
};
const demo_map = {
    sectors: [0x0, 0x3A, 0x3A, 0x3A, 0x3A, 0x3A, 0x3A, 0x191, 0x191, 0x47D, 0x630, 0x630, 0x630, 0x7EB, 0x9AC, 0xB68, 0xD1B, 0xD1B, 0xD1B, 0xEBB, 0xEBB, 0xEBB, 0xEBB, 0xEBB, 0xEBB, 0xEBB, 0xEBB, 0xEBB, 0xEBB, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x1046, 0x11F9, 0x11F9, 0x11F9, 0x13AD, 0x1564, 0x1713, 0x18C6, 0x18C6, 0x18C6, 0x1A66, 0x1A66, 0x1A66, 0x1A66, 0x1A66, 0x1A66, 0x1A66, 0x1A66, 0x1A66, 0x1A66, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x1BF1, 0x32FB, 0x3FFA, 0x4447, 0x4447, 0x4447, 0x4BB4, 0x4ED5, 0x50CA, 0x5517, 0x5517, 0x5928, 0x5928, 0x5928, 0x5928, 0x5928, 0x5928, 0x5928, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D39, 0x5D60, 0x5D88, 0x5D9E, 0x5DB8, 0x5DDF, 0x5E14, 0x5E33, 0x5E5B, 0x5E7B, 0x5E90, 0x5EB0, 0x5ED8, 0x5EFE, 0x5F1D, 0x5F39, 0x5F64, 0x5F7B, 0x5F93, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x5FA4, 0x6000, 0x6014, 0x602B, 0x602B, 0x602B, 0x602B, 0x605F, 0x6078, 0x6078, 0x607A, 0x607C, 0x607E, 0x6080, 0x60B6, 0x60E8, 0x6119, 0x6144, 0x617D, 0x6180, 0x6180, 0x6180, 0x6180, 0x6180, 0x6180, 0x61B8, 0x61EF, 0x61EF, 0x61EF, 0x61EF, 0x61EF, 0x61EF, 0x61EF, 0x621E, 0x621E, 0x621E, 0x621E, 0x621E, 0x621E, 0x621E, 0x621E, 0x621E, 0x621E, 0x621E, 0x621E, 0x621E, 0x621E, 0x621E, 0x621F, 0x621F, 0x621F, 0x6220, 0x6220, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6221, 0x6226, 0x622A, 0x622E, 0x6233, 0x6239, 0x623D, 0x6253, 0x6266, 0x6282, 0x62A2, 0x62C5, 0x62EA, 0x6309, 0x632D, 0x634B, 0x6368, 0x6386, 0x63A4, 0x63C2, 0x63E0, 0x63FE, 0x6421, 0x642A, 0x644D, 0x6456, 0x645F, 0x6468, 0x6471, 0x647A, 0x6483, 0x648C, 0x6495, 0x649E, 0x64A7, 0x64B0, 0x64B9, 0x64EC, 0x64EE, 0x6718, 0x6719, 0x6780, 0x6781, 0x67E5, 0x67E6, 0x6A1D, 0x6A1E, 0x6AD1, 0x6AD2, 0x6B7A, 0x6C1C, 0x6CC8, 0x6D70, 0x6E14, 0x6EC6, 0x6F75, 0x7035, 0x70D9, 0x71A9, 0x7282, 0x7357, 0x742F, 0x7513, 0x75EB, 0x76DB, 0x7776, 0x7777, 0x7778, 0x7779, 0x777A, 0x777B, 0x777C, 0x78D6, 0x78D7, 0x7921, 0x7927, 0x79C9, 0x79D6, 0x7B27, 0x7B54, 0x7C05, 0x7CF0, 0x7DAC, 0x7E3C, 0x7E97, 0x7EFD, 0x7F8B, 0x8069, 0x8147, 0x81A7, 0x81AD, 0x81B4, 0x81BC, 0x81CE, 0x81DD, 0x81E9, 0x81F3, 0x81F8, 0x81FF, 0x8210, 0x8226, 0x8231, 0x8240, 0x824F, 0x8267, 0x8272, 0x827D, 0x828C, 0x82A2, 0x82B1, 0x82C0, 0x82CD, 0x82D2, 0x82E5, 0x82F0, 0x82FE, 0x830D, 0x831D, 0x8324, 0x8335, 0x834A, 0x8354, 0x9CDC, 0x9D5D, 0x9DDE, 0x9E5F, 0x9EE0, 0x9F61, 0x9FE2, 0xA063, 0xA0E4, 0xA165, 0xA1E6, 0xA267, 0xA2E8, 0xA369, 0xA3EA, 0xA46B, 0xA4EC, 0xA56D, 0xA5AE, 0xA62F, 0xA6B0, 0xA731, 0xA7B2, 0xA833, 0xA8B4, 0xA935, 0xA9B6, 0xAA37, 0xAAB8, 0xAB39, 0xABBA, 0xAC3B, 0xACBC, 0xAD3D, 0xADBE, 0xAE3F, 0xAEC0, 0xAF41, 0xAFC2, 0xB043, 0xB0C4, 0xB145, 0xB1C6, 0xB247, 0xB2C8, 0xB349, 0xB3CA, 0xB44B, 0xB4CC, 0xB54D, 0xB5CE, 0xB64F, 0xB6D0, 0xB751, 0xB7D2, 0xB853, 0xB8D4, 0xB955, 0xB9D6, 0xBA57, 0xBAD8, 0xBB59, 0xBBDA, 0xBC5B, 0xBCDC, 0xBD5D, 0xBDDE, 0xBE5F, 0xBEE0, 0xBF61, 0xBFE2, 0xC063, 0xC0E4, 0xC165, 0xC1E6, 0xC267, 0xC2E8, 0xC369, 0xC3EA, 0xC46B, 0xC4EC, 0xC56D, 0xC5EE, 0xC66F, 0xC6F0, 0xC771, 0xC7F2, 0xC873, 0xC8F4, 0xC975, 0xC9F6, 0xCA77, 0xCAF8, 0xCB79, 0xCBFA, 0xCC7B, 0xCCFC, 0xCD7D, 0xCD7D, 0xCD7D, 0xCD7D, 0xCD7D, 0xCD7D, 0xCD7D, 0xCD7D, 0xCD7D, 0xCD7D, 0xD2A5, 0xD2A6, 0xD2A7, 0xD2C4, 0xD46E, 0xD48E, 0xD4E1, 0xD5A1, 0xD5A2, 0xD5A3, 0xD5A7, 0xD5AD, 0xD5AE, 0xD5AF, 0xD5B0, 0xD5B1, 0xD5B2, 0xD5B3, 0xD5B4, 0xD616, 0xD697, 0xD718, 0xD79A, 0xD7DC, 0xD7DC, 0xD7DC, 0xD7DC, 0xD7DC, 0xD7DC, 0xD7DC, 0xD7DC, 0xD7DC, 0xD7DC, 0xD7DC, 0xD7DC, 0xD7DC, 0xD7DC, 0xD7DC, 0xD7DC, 0xD7DC, 0xD7DC, 0xD7DC, 0xD7DC, 0xD7DC, 0xE84E, 0xE84F, 0xE850, 0xE86A, 0xEE48, 0xEE63, 0xEE9A, 0xEFB1, 0xEFB2, 0xEFB6, 0xEFCD, 0xEFD3, 0xEFD4, 0xEFD6, 0xEFD8, 0xEFD9, 0xEFDC, 0xEFDD, 0xEFDE, 0xF2DD, 0xF2DF, 0xF796, 0xF818, 0xF899, 0xF8DB, 0xF8DB, 0x10C52, 0x10C53, 0x10C54, 0x10C77, 0x112F1, 0x1131E, 0x1144E, 0x117A1, 0x117A2, 0x117A8, 0x117C1, 0x117D2, 0x117D3, 0x117D4, 0x117D7, 0x117D8, 0x117DB, 0x117E0, 0x117E1, 0x117E2, 0x117ED, 0x117EE, 0x11DB2, 0x11E34, 0x11E95, 0x11EF6, 0x11F78, 0x11FDA, 0x11FDA, 0x11FDA, 0x11FDA, 0x1226A, 0x123E9, 0x123EA, 0x123EB, 0x123F8, 0x12526, 0x12528, 0x12530, 0x12541, 0x12542, 0x12552, 0x12553, 0x12554, 0x12555, 0x12557, 0x12558, 0x12559, 0x1261E, 0x1261F, 0x12620, 0x12682, 0x12682, 0x12682, 0x12682, 0x12682, 0x12682, 0x12682, 0x12682, 0x12682, 0x12682, 0x12876, 0x12877, 0x12878, 0x1287D, 0x12996, 0x12997, 0x129A7, 0x129C0, 0x129C1, 0x129C3, 0x129C5, 0x129C7, 0x129C8, 0x129C9, 0x129CA, 0x129CB, 0x129CD, 0x129CE, 0x129CF, 0x129D0, 0x129D6, 0x129D7, 0x12A18, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79, 0x12A79],
    sizes: [0x3A, 0x0, 0x0, 0x0, 0x0, 0x0, 0x157, 0x0, 0x2EC, 0x1B3, 0x0, 0x0, 0x1BB, 0x1C1, 0x1BC, 0x1B3, 0x0, 0x0, 0x1A0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x18B, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1B3, 0x0, 0x0, 0x1B4, 0x1B7, 0x1AF, 0x1B3, 0x0, 0x0, 0x1A0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x18B, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x170A, 0xCFF, 0x44D, 0x0, 0x0, 0x76D, 0x321, 0x1F5, 0x44D, 0x0, 0x411, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x411, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x27, 0x28, 0x16, 0x1A, 0x27, 0x35, 0x1F, 0x28, 0x20, 0x15, 0x20, 0x28, 0x26, 0x1F, 0x1C, 0x2B, 0x17, 0x18, 0x11, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x5C, 0x14, 0x17, 0x0, 0x0, 0x0, 0x34, 0x19, 0x0, 0x2, 0x2, 0x2, 0x2, 0x36, 0x32, 0x31, 0x2B, 0x39, 0x3, 0x0, 0x0, 0x0, 0x0, 0x0, 0x38, 0x37, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2F, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x1, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x5, 0x4, 0x4, 0x5, 0x6, 0x4, 0x16, 0x13, 0x1C, 0x20, 0x23, 0x25, 0x1F, 0x24, 0x1E, 0x1D, 0x1E, 0x1E, 0x1E, 0x1E, 0x1E, 0x23, 0x9, 0x23, 0x9, 0x9, 0x9, 0x9, 0x9, 0x9, 0x9, 0x9, 0x9, 0x9, 0x9, 0x9, 0x33, 0x2, 0x22A, 0x1, 0x67, 0x1, 0x64, 0x1, 0x237, 0x1, 0xB3, 0x1, 0xA8, 0xA2, 0xAC, 0xA8, 0xA4, 0xB2, 0xAF, 0xC0, 0xA4, 0xD0, 0xD9, 0xD5, 0xD8, 0xE4, 0xD8, 0xF0, 0x9B, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x15A, 0x1, 0x4A, 0x6, 0xA2, 0xD, 0x151, 0x2D, 0xB1, 0xEB, 0xBC, 0x90, 0x5B, 0x66, 0x8E, 0xDE, 0xDE, 0x60, 0x6, 0x7, 0x8, 0x12, 0xF, 0xC, 0xA, 0x5, 0x7, 0x11, 0x16, 0xB, 0xF, 0xF, 0x18, 0xB, 0xB, 0xF, 0x16, 0xF, 0xF, 0xD, 0x5, 0x13, 0xB, 0xE, 0xF, 0x10, 0x7, 0x11, 0x15, 0xA, 0x1988, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x41, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x528, 0x1, 0x1, 0x1D, 0x1AA, 0x20, 0x53, 0xC0, 0x1, 0x1, 0x4, 0x6, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x62, 0x81, 0x81, 0x82, 0x42, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1072, 0x1, 0x1, 0x1A, 0x5DE, 0x1B, 0x37, 0x117, 0x1, 0x4, 0x17, 0x6, 0x1, 0x2, 0x2, 0x1, 0x3, 0x1, 0x1, 0x2FF, 0x2, 0x4B7, 0x82, 0x81, 0x42, 0x0, 0x1377, 0x1, 0x1, 0x23, 0x67A, 0x2D, 0x130, 0x353, 0x1, 0x6, 0x19, 0x11, 0x1, 0x1, 0x3, 0x1, 0x3, 0x5, 0x1, 0x1, 0xB, 0x1, 0x5C4, 0x82, 0x61, 0x61, 0x82, 0x62, 0x0, 0x0, 0x0, 0x290, 0x17F, 0x1, 0x1, 0xD, 0x12E, 0x2, 0x8, 0x11, 0x1, 0x10, 0x1, 0x1, 0x1, 0x2, 0x1, 0x1, 0xC5, 0x1, 0x1, 0x62, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1F4, 0x1, 0x1, 0x5, 0x119, 0x1, 0x10, 0x19, 0x1, 0x2, 0x2, 0x2, 0x1, 0x1, 0x1, 0x1, 0x2, 0x1, 0x1, 0x1, 0x6, 0x1, 0x41, 0x61, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0]
};
